# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.modules.etcd_mod as etcd_mod
import salt.utils.etcd_util as etcd_util

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, create_autospec, patch
from tests.support.unit import TestCase


class EtcdModTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.etcd_mod
    """

    def setup_loader_modules(self):
        return {etcd_mod: {}}

    def setUp(self):
        self.instance = create_autospec(etcd_util.EtcdClient)
        self.EtcdClientMock = MagicMock()
        self.EtcdClientMock.return_value = self.instance

    def tearDown(self):
        del self.instance
        del self.EtcdClientMock

    # 'get_' function tests: 1

    def test_get(self):
        """
        Test if it get a value from etcd, by direct path
        """
        with patch.dict(
            etcd_mod.__utils__, {"etcd_util.get_conn": self.EtcdClientMock}
        ):
            self.instance.get.return_value = "stack"
            self.assertEqual(etcd_mod.get_("salt"), "stack")
            self.instance.get.assert_called_with("salt", recurse=False)

            self.instance.tree.return_value = {}
            self.assertEqual(etcd_mod.get_("salt", recurse=True), {})
            self.instance.tree.assert_called_with("salt")

            self.instance.get.side_effect = Exception
            self.assertRaises(Exception, etcd_mod.get_, "err")

    # 'set_' function tests: 1

    def test_set(self):
        """
        Test if it set a key in etcd, by direct path
        """
        with patch.dict(
            etcd_mod.__utils__, {"etcd_util.get_conn": self.EtcdClientMock}
        ):
            self.instance.set.return_value = "stack"
            self.assertEqual(etcd_mod.set_("salt", "stack"), "stack")
            self.instance.set.assert_called_with(
                "salt", "stack", directory=False, ttl=None
            )

            self.instance.set.return_value = True
            self.assertEqual(etcd_mod.set_("salt", "", directory=True), True)
            self.instance.set.assert_called_with("salt", "", directory=True, ttl=None)

            self.assertEqual(etcd_mod.set_("salt", "", directory=True, ttl=5), True)
            self.instance.set.assert_called_with("salt", "", directory=True, ttl=5)

            self.assertEqual(etcd_mod.set_("salt", "", None, 10, True), True)
            self.instance.set.assert_called_with("salt", "", directory=True, ttl=10)

            self.instance.set.side_effect = Exception
            self.assertRaises(Exception, etcd_mod.set_, "err", "stack")

    # 'update' function tests: 1

    def test_update(self):
        """
        Test if can set multiple keys in etcd
        """
        with patch.dict(
            etcd_mod.__utils__, {"etcd_util.get_conn": self.EtcdClientMock}
        ):
            args = {
                "x": {"y": {"a": "1", "b": "2"}},
                "z": "4",
                "d": {},
            }

            result = {
                "/some/path/x/y/a": "1",
                "/some/path/x/y/b": "2",
                "/some/path/z": "4",
                "/some/path/d": {},
            }
            self.instance.update.return_value = result
            self.assertDictEqual(etcd_mod.update(args, path="/some/path"), result)
            self.instance.update.assert_called_with(args, "/some/path")
            self.assertDictEqual(etcd_mod.update(args), result)
            self.instance.update.assert_called_with(args, "")

    # 'ls_' function tests: 1

    def test_ls(self):
        """
        Test if it return all keys and dirs inside a specific path
        """
        with patch.dict(
            etcd_mod.__utils__, {"etcd_util.get_conn": self.EtcdClientMock}
        ):
            self.instance.ls.return_value = {"/some-dir": {}}
            self.assertDictEqual(etcd_mod.ls_("/some-dir"), {"/some-dir": {}})
            self.instance.ls.assert_called_with("/some-dir")

            self.instance.ls.return_value = {"/": {}}
            self.assertDictEqual(etcd_mod.ls_(), {"/": {}})
            self.instance.ls.assert_called_with("/")

            self.instance.ls.side_effect = Exception
            self.assertRaises(Exception, etcd_mod.ls_, "err")

    # 'rm_' function tests: 1

    def test_rm(self):
        """
        Test if it delete a key from etcd
        """
        with patch.dict(
            etcd_mod.__utils__, {"etcd_util.get_conn": self.EtcdClientMock}
        ):
            self.instance.rm.return_value = False
            self.assertFalse(etcd_mod.rm_("dir"))
            self.instance.rm.assert_called_with("dir", recurse=False)

            self.instance.rm.return_value = True
            self.assertTrue(etcd_mod.rm_("dir", recurse=True))
            self.instance.rm.assert_called_with("dir", recurse=True)

            self.instance.rm.side_effect = Exception
            self.assertRaises(Exception, etcd_mod.rm_, "err")

    # 'tree' function tests: 1

    def test_tree(self):
        """
        Test if it recurses through etcd and return all values
        """
        with patch.dict(
            etcd_mod.__utils__, {"etcd_util.get_conn": self.EtcdClientMock}
        ):
            self.instance.tree.return_value = {}
            self.assertDictEqual(etcd_mod.tree("/some-dir"), {})
            self.instance.tree.assert_called_with("/some-dir")

            self.assertDictEqual(etcd_mod.tree(), {})
            self.instance.tree.assert_called_with("/")

            self.instance.tree.side_effect = Exception
            self.assertRaises(Exception, etcd_mod.tree, "err")

    # 'watch' function tests: 1

    def test_watch(self):
        """
        Test if watch returns the right tuples
        """
        with patch.dict(
            etcd_mod.__utils__, {"etcd_util.get_conn": self.EtcdClientMock}
        ):
            self.instance.watch.return_value = {
                "value": "stack",
                "changed": True,
                "dir": False,
                "mIndex": 1,
                "key": "/salt",
            }
            self.assertEqual(etcd_mod.watch("/salt"), self.instance.watch.return_value)
            self.instance.watch.assert_called_with(
                "/salt", recurse=False, timeout=0, index=None
            )

            self.instance.watch.return_value["dir"] = True
            self.assertEqual(
                etcd_mod.watch("/some-dir", recurse=True, timeout=5, index=10),
                self.instance.watch.return_value,
            )
            self.instance.watch.assert_called_with(
                "/some-dir", recurse=True, timeout=5, index=10
            )

            self.assertEqual(
                etcd_mod.watch("/some-dir", True, None, 5, 10),
                self.instance.watch.return_value,
            )
            self.instance.watch.assert_called_with(
                "/some-dir", recurse=True, timeout=5, index=10
            )



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import

import textwrap

# Import Salt Libs
import salt.modules.pkgng as pkgng

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class ListPackages(object):
    def __init__(self):
        self._iteration = 0

    def __call__(self):
        pkg_lists = [
            {"openvpn": "2.4.8_2"},
            {
                "gettext-runtime": "0.20.1",
                "openvpn": "2.4.8_2",
                "p5-Mojolicious": "8.40",
            },
        ]
        pkgs = pkg_lists[self._iteration]
        self._iteration += 1
        return pkgs


class PkgNgTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.pkgng
    """

    @classmethod
    def setup_loader_modules(cls):
        return {pkgng: {"__salt__": {}}}

    def test_lock(self):
        """
        Test pkgng.lock
        """
        lock_cmd = MagicMock(
            return_value={"stdout": ("pkga-1.0\n" "pkgb-2.0\n"), "retcode": 0}
        )
        with patch.dict(pkgng.__salt__, {"cmd.run_all": lock_cmd}):

            result = pkgng.lock("pkga")
            self.assertTrue(result)
            lock_cmd.assert_called_with(
                ["pkg", "lock", "-y", "--quiet", "--show-locked", "pkga"],
                output_loglevel="trace",
                python_shell=False,
            )

            result = pkgng.lock("dummy")
            self.assertFalse(result)
            lock_cmd.assert_called_with(
                ["pkg", "lock", "-y", "--quiet", "--show-locked", "dummy"],
                output_loglevel="trace",
                python_shell=False,
            )

    def test_unlock(self):
        """
        Test pkgng.unlock
        """
        unlock_cmd = MagicMock(
            return_value={"stdout": ("pkga-1.0\n" "pkgb-2.0\n"), "retcode": 0}
        )
        with patch.dict(pkgng.__salt__, {"cmd.run_all": unlock_cmd}):

            result = pkgng.unlock("pkga")
            self.assertFalse(result)
            unlock_cmd.assert_called_with(
                ["pkg", "unlock", "-y", "--quiet", "--show-locked", "pkga"],
                output_loglevel="trace",
                python_shell=False,
            )

            result = pkgng.unlock("dummy")
            self.assertTrue(result)
            unlock_cmd.assert_called_with(
                ["pkg", "unlock", "-y", "--quiet", "--show-locked", "dummy"],
                output_loglevel="trace",
                python_shell=False,
            )

    def test_locked(self):
        """
        Test pkgng.unlock
        """
        lock_cmd = MagicMock(
            return_value={"stdout": ("pkga-1.0\n" "pkgb-2.0\n"), "retcode": 0}
        )
        with patch.dict(pkgng.__salt__, {"cmd.run_all": lock_cmd}):

            result = pkgng.locked("pkga")
            self.assertTrue(result)
            lock_cmd.assert_called_with(
                ["pkg", "lock", "-y", "--quiet", "--show-locked"],
                output_loglevel="trace",
                python_shell=False,
            )

            result = pkgng.locked("dummy")
            self.assertFalse(result)
            lock_cmd.assert_called_with(
                ["pkg", "lock", "-y", "--quiet", "--show-locked"],
                output_loglevel="trace",
                python_shell=False,
            )

    def test_list_upgrades_present(self):
        """
        Test pkgng.list_upgrades with upgrades available
        """
        pkg_cmd = MagicMock(
            return_value=textwrap.dedent(
                """
            The following 6 package(s) will be affected (of 0 checked):

            Installed packages to be UPGRADED:
                    pkga: 1.0 -> 1.1
                    pkgb: 2.0 -> 2.1 [FreeBSD]
                    pkgc: 3.0 -> 3.1 [FreeBSD] (dependency changed)
                    pkgd: 4.0 -> 4.1 (dependency changed)

            New packages to be INSTALLED:
                    pkge: 1.0
                    pkgf: 2.0 [FreeBSD]
                    pkgg: 3.0 [FreeBSD] (dependency changed)
                    pkgh: 4.0 (dependency changed)

            Installed packages to be REINSTALLED:
                    pkgi-1.0
                    pkgj-2.0 [FreeBSD]
                    pkgk-3.0 [FreeBSD] (direct dependency changed: pkga)
                    pkgl-4.0 (direct dependency changed: pkgb)

            Installed packages to be DOWNGRADED:
                    pkgm: 1.1 -> 1.0
                    pkgn: 2.1 -> 2.0 [FreeBSD]
                    pkgo: 3.1 -> 3.0 [FreeBSD] (dependency changed)
                    pkgp: 4.1 -> 4.0 (dependency changed)

            Installed packages to be REMOVED:
                    pkgq-1.0
                    pkgr-2.0 [FreeBSD]
                    pkgs-3.0 [FreeBSD] (direct dependency changed: pkga)
                    pkgt-4.0 (direct dependency changed: pkgb)

            Number of packages to be upgraded: 2
            Number of packages to be reinstalled: 2

            The process will require 14 MiB more space.
            22 MiB to be downloaded.
            """
            )
        )

        with patch.dict(pkgng.__salt__, {"cmd.run_stdout": pkg_cmd}):

            result = pkgng.list_upgrades(refresh=False)
            self.assertDictEqual(
                result, {"pkga": "1.1", "pkgb": "2.1", "pkgc": "3.1", "pkgd": "4.1"}
            )
            pkg_cmd.assert_called_with(
                ["pkg", "upgrade", "--dry-run", "--quiet", "--no-repo-update"],
                output_loglevel="trace",
                python_shell=False,
                ignore_retcode=True,
            )

    def test_list_upgrades_absent(self):
        """
        Test pkgng.list_upgrades with no upgrades available
        """
        pkg_cmd = MagicMock(return_value="")

        with patch.dict(pkgng.__salt__, {"cmd.run_stdout": pkg_cmd}):
            result = pkgng.list_upgrades(refresh=False)
            self.assertDictEqual(result, {})
            pkg_cmd.assert_called_with(
                ["pkg", "upgrade", "--dry-run", "--quiet", "--no-repo-update"],
                output_loglevel="trace",
                python_shell=False,
                ignore_retcode=True,
            )

    def test_upgrade_without_fromrepo(self):
        """
        Test pkg upgrade to upgrade all available packages
        """
        pkg_cmd = MagicMock(return_value={"retcode": 0})

        with patch.dict(pkgng.__salt__, {"cmd.run_all": pkg_cmd}):
            with patch("salt.modules.pkgng.list_pkgs", ListPackages()):
                result = pkgng.upgrade()
                expected = {
                    "gettext-runtime": {"new": "0.20.1", "old": ""},
                    "p5-Mojolicious": {"new": "8.40", "old": ""},
                }
                self.assertDictEqual(result, expected)
                pkg_cmd.assert_called_with(
                    ["pkg", "upgrade", "-y"],
                    output_loglevel="trace",
                    python_shell=False,
                )

    def test_upgrade_with_fromrepo(self):
        """
        Test pkg upgrade to upgrade all available packages
        """
        pkg_cmd = MagicMock(return_value={"retcode": 0})

        with patch.dict(pkgng.__salt__, {"cmd.run_all": pkg_cmd}):
            with patch("salt.modules.pkgng.list_pkgs", ListPackages()):
                result = pkgng.upgrade(fromrepo="FreeBSD")
                expected = {
                    "gettext-runtime": {"new": "0.20.1", "old": ""},
                    "p5-Mojolicious": {"new": "8.40", "old": ""},
                }
                self.assertDictEqual(result, expected)
                pkg_cmd.assert_called_with(
                    ["pkg", "upgrade", "-y", "--repository", "FreeBSD"],
                    output_loglevel="trace",
                    python_shell=False,
                )

    def test_upgrade_with_fetchonly(self):
        """
        Test pkg upgrade to fetch packages only
        """
        pkg_cmd = MagicMock(return_value={"retcode": 0})

        with patch.dict(pkgng.__salt__, {"cmd.run_all": pkg_cmd}):
            with patch("salt.modules.pkgng.list_pkgs", ListPackages()):
                result = pkgng.upgrade(fetchonly=True)
                expected = {
                    "gettext-runtime": {"new": "0.20.1", "old": ""},
                    "p5-Mojolicious": {"new": "8.40", "old": ""},
                }
                self.assertDictEqual(result, expected)
                pkg_cmd.assert_called_with(
                    ["pkg", "upgrade", "-Fy"],
                    output_loglevel="trace",
                    python_shell=False,
                )

    def test_stats_with_local(self):
        """
        Test pkg.stats for local packages
        """
        pkg_cmd = MagicMock(return_value="")

        with patch.dict(pkgng.__salt__, {"cmd.run": pkg_cmd}):
            result = pkgng.stats(local=True)
            self.assertEqual(result, [])
            pkg_cmd.assert_called_with(
                ["pkg", "stats", "-l"], output_loglevel="trace", python_shell=False,
            )

    def test_stats_with_remote(self):
        """
        Test pkg.stats for remote packages
        """
        pkg_cmd = MagicMock(return_value="")

        with patch.dict(pkgng.__salt__, {"cmd.run": pkg_cmd}):
            result = pkgng.stats(remote=True)
            self.assertEqual(result, [])
            pkg_cmd.assert_called_with(
                ["pkg", "stats", "-r"], output_loglevel="trace", python_shell=False,
            )



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

import uuid

# Import Salt Libs
import salt.modules.iptables as iptables

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class IptablesTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.iptables
    """

    def setup_loader_modules(self):
        return {iptables: {}}

    # 'version' function tests: 1

    def test_version(self):
        """
        Test if it return version from iptables --version
        """
        mock = MagicMock(return_value="iptables v1.4.21")
        with patch.dict(iptables.__salt__, {"cmd.run": mock}):
            self.assertEqual(iptables.version(), "v1.4.21")

    # 'build_rule' function tests: 1

    def test_build_rule(self):
        """
        Test if it build a well-formatted iptables rule based on kwargs.
        """
        with patch.object(iptables, "_has_option", MagicMock(return_value=True)):
            self.assertEqual(iptables.build_rule(), "")

            self.assertEqual(
                iptables.build_rule(name="ignored", state="ignored"),
                "",
                "build_rule should ignore name and state",
            )

            # Should properly negate bang-prefixed values
            self.assertEqual(iptables.build_rule(**{"if": "!eth0"}), "! -i eth0")

            # Should properly negate "not"-prefixed values
            self.assertEqual(iptables.build_rule(**{"if": "not eth0"}), "! -i eth0")

            self.assertEqual(
                iptables.build_rule(**{"protocol": "tcp", "syn": "!"}), "-p tcp ! --syn"
            )

            self.assertEqual(
                iptables.build_rule(dports=[80, 443], protocol="tcp"),
                "-p tcp -m multiport --dports 80,443",
            )

            self.assertEqual(
                iptables.build_rule(dports="80,443", protocol="tcp"),
                "-p tcp -m multiport --dports 80,443",
            )

            # Should it really behave this way?
            self.assertEqual(
                iptables.build_rule(dports=["!80", 443], protocol="tcp"),
                "-p tcp -m multiport ! --dports 80,443",
            )

            self.assertEqual(
                iptables.build_rule(dports="!80,443", protocol="tcp"),
                "-p tcp -m multiport ! --dports 80,443",
            )

            self.assertEqual(
                iptables.build_rule(sports=[80, 443], protocol="tcp"),
                "-p tcp -m multiport --sports 80,443",
            )

            self.assertEqual(
                iptables.build_rule(sports="80,443", protocol="tcp"),
                "-p tcp -m multiport --sports 80,443",
            )

            self.assertEqual(
                iptables.build_rule(
                    "filter",
                    "INPUT",
                    command="I",
                    position="3",
                    full=True,
                    dports="protocol",
                    jump="ACCEPT",
                ),
                "Error: protocol must be specified",
            )

            self.assertEqual(
                iptables.build_rule(
                    "filter",
                    "INPUT",
                    command="I",
                    position="3",
                    full=True,
                    sports="protocol",
                    jump="ACCEPT",
                ),
                "Error: protocol must be specified",
            )

            self.assertEqual(
                iptables.build_rule(
                    "",
                    "INPUT",
                    command="I",
                    position="3",
                    full="True",
                    match="state",
                    jump="ACCEPT",
                ),
                "Error: Table needs to be specified",
            )

            self.assertEqual(
                iptables.build_rule(
                    "filter",
                    "",
                    command="I",
                    position="3",
                    full="True",
                    match="state",
                    jump="ACCEPT",
                ),
                "Error: Chain needs to be specified",
            )

            self.assertEqual(
                iptables.build_rule(
                    "filter",
                    "INPUT",
                    command="",
                    position="3",
                    full="True",
                    match="state",
                    jump="ACCEPT",
                ),
                "Error: Command needs to be specified",
            )

            # Test arguments that should appear after the --jump
            self.assertEqual(
                iptables.build_rule(jump="REDIRECT", **{"to-port": 8080}),
                "--jump REDIRECT --to-port 8080",
            )

            # Should quote arguments with spaces, like log-prefix often has
            self.assertEqual(
                iptables.build_rule(jump="LOG", **{"log-prefix": "long prefix"}),
                '--jump LOG --log-prefix "long prefix"',
            )

            # Should quote arguments with leading or trailing spaces
            self.assertEqual(
                iptables.build_rule(jump="LOG", **{"log-prefix": "spam: "}),
                '--jump LOG --log-prefix "spam: "',
            )

            # Should allow no-arg jump options
            self.assertEqual(
                iptables.build_rule(jump="CLUSTERIP", **{"new": ""}),
                "--jump CLUSTERIP --new",
            )

            # Should allow no-arg jump options as None
            self.assertEqual(
                iptables.build_rule(jump="CT", **{"notrack": None}),
                "--jump CT --notrack",
            )

            # should build match-sets with single string
            self.assertEqual(
                iptables.build_rule(**{"match-set": "src flag1,flag2"}),
                "-m set --match-set src flag1,flag2",
            )

            # should build match-sets as list
            match_sets = [
                "src1 flag1",
                "src2 flag2,flag3",
            ]
            self.assertEqual(
                iptables.build_rule(**{"match-set": match_sets}),
                "-m set --match-set src1 flag1 -m set --match-set src2 flag2,flag3",
            )

            # should handle negations for string match-sets
            self.assertEqual(
                iptables.build_rule(**{"match-set": "!src flag"}),
                "-m set ! --match-set src flag",
            )

            # should handle negations for list match-sets
            match_sets = ["src1 flag", "not src2 flag2"]
            self.assertEqual(
                iptables.build_rule(**{"match-set": match_sets}),
                "-m set --match-set src1 flag -m set ! --match-set src2 flag2",
            )

            # should allow escaped name
            self.assertEqual(
                iptables.build_rule(**{"match": "recent", "name_": "SSH"}),
                "-m recent --name SSH",
            )

            # should allow empty arguments
            self.assertEqual(
                iptables.build_rule(**{"match": "recent", "update": None}),
                "-m recent --update",
            )

            # Should allow the --save jump option to CONNSECMARK
            # self.assertEqual(iptables.build_rule(jump='CONNSECMARK',
            #                                     **{'save': ''}),
            #                 '--jump CONNSECMARK --save ')

            ret = "/sbin/iptables --wait -t salt -I INPUT 3 -m state --jump ACCEPT"
            with patch.object(
                iptables, "_iptables_cmd", MagicMock(return_value="/sbin/iptables")
            ):
                self.assertEqual(
                    iptables.build_rule(
                        "salt",
                        "INPUT",
                        command="I",
                        position="3",
                        full="True",
                        match="state",
                        jump="ACCEPT",
                    ),
                    ret,
                )

    # 'get_saved_rules' function tests: 1

    def test_get_saved_rules(self):
        """
        Test if it return a data structure of the rules in the conf file
        """
        mock = MagicMock(return_value=False)
        with patch.object(iptables, "_parse_conf", mock):
            self.assertFalse(iptables.get_saved_rules())
            mock.assert_called_with(conf_file=None, family="ipv4")

    # 'get_rules' function tests: 1

    def test_get_rules(self):
        """
        Test if it return a data structure of the current, in-memory rules
        """
        mock = MagicMock(return_value=False)
        with patch.object(iptables, "_parse_conf", mock):
            self.assertFalse(iptables.get_rules())
            mock.assert_called_with(in_mem=True, family="ipv4")

    # 'get_saved_policy' function tests: 1

    def test_get_saved_policy(self):
        """
        Test if it return the current policy for the specified table/chain
        """
        self.assertEqual(
            iptables.get_saved_policy(
                table="filter", chain=None, conf_file=None, family="ipv4"
            ),
            "Error: Chain needs to be specified",
        )

        with patch.object(
            iptables,
            "_parse_conf",
            MagicMock(return_value={"filter": {"INPUT": {"policy": True}}}),
        ):
            self.assertTrue(
                iptables.get_saved_policy(
                    table="filter", chain="INPUT", conf_file=None, family="ipv4"
                )
            )

        with patch.object(
            iptables,
            "_parse_conf",
            MagicMock(return_value={"filter": {"INPUT": {"policy1": True}}}),
        ):
            self.assertIsNone(
                iptables.get_saved_policy(
                    table="filter", chain="INPUT", conf_file=None, family="ipv4"
                )
            )

    # 'get_policy' function tests: 1

    def test_get_policy(self):
        """
        Test if it return the current policy for the specified table/chain
        """
        self.assertEqual(
            iptables.get_policy(table="filter", chain=None, family="ipv4"),
            "Error: Chain needs to be specified",
        )

        with patch.object(
            iptables,
            "_parse_conf",
            MagicMock(return_value={"filter": {"INPUT": {"policy": True}}}),
        ):
            self.assertTrue(
                iptables.get_policy(table="filter", chain="INPUT", family="ipv4")
            )

        with patch.object(
            iptables,
            "_parse_conf",
            MagicMock(return_value={"filter": {"INPUT": {"policy1": True}}}),
        ):
            self.assertIsNone(
                iptables.get_policy(table="filter", chain="INPUT", family="ipv4")
            )

    # 'set_policy' function tests: 1

    def test_set_policy(self):
        """
        Test if it set the current policy for the specified table/chain
        """
        with patch.object(iptables, "_has_option", MagicMock(return_value=True)):
            self.assertEqual(
                iptables.set_policy(
                    table="filter", chain=None, policy=None, family="ipv4"
                ),
                "Error: Chain needs to be specified",
            )

            self.assertEqual(
                iptables.set_policy(
                    table="filter", chain="INPUT", policy=None, family="ipv4"
                ),
                "Error: Policy needs to be specified",
            )

            mock = MagicMock(return_value=True)
            with patch.dict(iptables.__salt__, {"cmd.run": mock}):
                self.assertTrue(
                    iptables.set_policy(
                        table="filter", chain="INPUT", policy="ACCEPT", family="ipv4"
                    )
                )

    # 'save' function tests: 1

    def test_save(self):
        """
        Test if it save the current in-memory rules to disk
        """
        with patch("salt.modules.iptables._conf", MagicMock(return_value=False)), patch(
            "os.path.isdir", MagicMock(return_value=True)
        ):
            mock = MagicMock(return_value=True)
            with patch.dict(
                iptables.__salt__,
                {
                    "cmd.run": mock,
                    "file.write": mock,
                    "config.option": MagicMock(return_value=[]),
                },
            ):
                self.assertTrue(iptables.save(filename="/xyz", family="ipv4"))

    # 'check' function tests: 1

    def test_check(self):
        """
        Test if it check for the existence of a rule in the table and chain
        """
        self.assertEqual(
            iptables.check(table="filter", chain=None, rule=None, family="ipv4"),
            "Error: Chain needs to be specified",
        )

        self.assertEqual(
            iptables.check(table="filter", chain="INPUT", rule=None, family="ipv4"),
            "Error: Rule needs to be specified",
        )

        mock_rule = "m state --state RELATED,ESTABLISHED -j ACCEPT"
        mock_chain = "INPUT"
        mock_uuid = 31337
        mock_cmd = MagicMock(
            return_value="-A {0}\n-A {1}".format(mock_chain, hex(mock_uuid))
        )
        mock_has = MagicMock(return_value=True)
        mock_not = MagicMock(return_value=False)

        with patch.object(iptables, "_has_option", mock_not):
            with patch.object(uuid, "getnode", MagicMock(return_value=mock_uuid)):
                with patch.dict(iptables.__salt__, {"cmd.run": mock_cmd}):
                    self.assertTrue(
                        iptables.check(
                            table="filter",
                            chain=mock_chain,
                            rule=mock_rule,
                            family="ipv4",
                        )
                    )

        mock_cmd = MagicMock(return_value="")

        with patch.object(iptables, "_has_option", mock_not):
            with patch.object(uuid, "getnode", MagicMock(return_value=mock_uuid)):
                with patch.dict(
                    iptables.__salt__, {"cmd.run": MagicMock(return_value="")}
                ):
                    self.assertFalse(
                        iptables.check(
                            table="filter",
                            chain=mock_chain,
                            rule=mock_rule,
                            family="ipv4",
                        )
                    )

        with patch.object(iptables, "_has_option", mock_has):
            with patch.dict(iptables.__salt__, {"cmd.run": mock_cmd}):
                self.assertTrue(
                    iptables.check(
                        table="filter", chain="INPUT", rule=mock_rule, family="ipv4"
                    )
                )

        mock_cmd = MagicMock(return_value="-A 0x4d2")
        mock_uuid = MagicMock(return_value=1234)

        with patch.object(iptables, "_has_option", mock_has):
            with patch.object(uuid, "getnode", mock_uuid):
                with patch.dict(iptables.__salt__, {"cmd.run": mock_cmd}):
                    self.assertTrue(
                        iptables.check(
                            table="filter", chain="0x4d2", rule=mock_rule, family="ipv4"
                        )
                    )

    # 'check_chain' function tests: 1

    def test_check_chain(self):
        """
        Test if it check for the existence of a chain in the table
        """
        self.assertEqual(
            iptables.check_chain(table="filter", chain=None, family="ipv4"),
            "Error: Chain needs to be specified",
        )

        mock_cmd = MagicMock(return_value="")
        with patch.dict(iptables.__salt__, {"cmd.run": mock_cmd}):
            self.assertFalse(
                iptables.check_chain(table="filter", chain="INPUT", family="ipv4")
            )

    # 'new_chain' function tests: 1

    def test_new_chain(self):
        """
        Test if it create new custom chain to the specified table.
        """
        self.assertEqual(
            iptables.new_chain(table="filter", chain=None, family="ipv4"),
            "Error: Chain needs to be specified",
        )

        mock_cmd = MagicMock(return_value="")
        with patch.dict(iptables.__salt__, {"cmd.run": mock_cmd}):
            self.assertTrue(
                iptables.new_chain(table="filter", chain="INPUT", family="ipv4")
            )

    # 'delete_chain' function tests: 1

    def test_delete_chain(self):
        """
        Test if it delete custom chain to the specified table.
        """
        self.assertEqual(
            iptables.delete_chain(table="filter", chain=None, family="ipv4"),
            "Error: Chain needs to be specified",
        )

        mock_cmd = MagicMock(return_value="")
        with patch.dict(iptables.__salt__, {"cmd.run": mock_cmd}):
            self.assertTrue(
                iptables.delete_chain(table="filter", chain="INPUT", family="ipv4")
            )

    # 'append' function tests: 1

    def test_append(self):
        """
        Test if it append a rule to the specified table/chain.
        """
        with patch.object(
            iptables, "_has_option", MagicMock(return_value=True)
        ), patch.object(iptables, "check", MagicMock(return_value=False)):
            self.assertEqual(
                iptables.append(table="filter", chain=None, rule=None, family="ipv4"),
                "Error: Chain needs to be specified",
            )

            self.assertEqual(
                iptables.append(
                    table="filter", chain="INPUT", rule=None, family="ipv4"
                ),
                "Error: Rule needs to be specified",
            )

            _rule = "m state --state RELATED,ESTABLISHED -j ACCEPT"
            mock = MagicMock(side_effect=["", "SALT"])
            with patch.dict(iptables.__salt__, {"cmd.run": mock}):
                self.assertTrue(
                    iptables.append(
                        table="filter", chain="INPUT", rule=_rule, family="ipv4"
                    )
                )

                self.assertFalse(
                    iptables.append(
                        table="filter", chain="INPUT", rule=_rule, family="ipv4"
                    )
                )

    # 'insert' function tests: 1

    def test_insert(self):
        """
        Test if it insert a rule into the specified table/chain,
        at the specified position.
        """
        with patch.object(
            iptables, "_has_option", MagicMock(return_value=True)
        ), patch.object(iptables, "check", MagicMock(return_value=False)):
            self.assertEqual(
                iptables.insert(
                    table="filter", chain=None, position=None, rule=None, family="ipv4"
                ),
                "Error: Chain needs to be specified",
            )

            pos_err = "Error: Position needs to be specified or use append (-A)"
            self.assertEqual(
                iptables.insert(
                    table="filter",
                    chain="INPUT",
                    position=None,
                    rule=None,
                    family="ipv4",
                ),
                pos_err,
            )

            self.assertEqual(
                iptables.insert(
                    table="filter", chain="INPUT", position=3, rule=None, family="ipv4"
                ),
                "Error: Rule needs to be specified",
            )

            _rule = "m state --state RELATED,ESTABLISHED -j ACCEPT"
            mock = MagicMock(return_value=True)
            with patch.dict(iptables.__salt__, {"cmd.run": mock}):
                self.assertTrue(
                    iptables.insert(
                        table="filter",
                        chain="INPUT",
                        position=3,
                        rule=_rule,
                        family="ipv4",
                    )
                )

    # 'delete' function tests: 1

    def test_delete(self):
        """
        Test if it delete a rule from the specified table/chain
        """
        with patch.object(iptables, "_has_option", MagicMock(return_value=True)):
            _rule = "m state --state RELATED,ESTABLISHED -j ACCEPT"
            self.assertEqual(
                iptables.delete(
                    table="filter", chain=None, position=3, rule=_rule, family="ipv4"
                ),
                "Error: Only specify a position or a rule, not both",
            )

            mock = MagicMock(return_value=True)
            with patch.dict(iptables.__salt__, {"cmd.run": mock}):
                self.assertTrue(
                    iptables.delete(
                        table="filter",
                        chain="INPUT",
                        position=3,
                        rule="",
                        family="ipv4",
                    )
                )

    # 'flush' function tests: 1

    def test_flush(self):
        """
        Test if it flush the chain in the specified table,
        flush all chains in the specified table if not specified chain.
        """
        with patch.object(iptables, "_has_option", MagicMock(return_value=True)):
            mock_cmd = MagicMock(return_value=True)
            with patch.dict(iptables.__salt__, {"cmd.run": mock_cmd}):
                self.assertTrue(
                    iptables.flush(table="filter", chain="INPUT", family="ipv4")
                )



# -*- coding: utf-8 -*-
"""
    :codeauthor: Pedro Algarvio (pedro@algarvio.me)


    tests.integration.states.pip_state
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""

from __future__ import absolute_import, print_function, unicode_literals

import errno
import glob
import os
import pprint
import shutil
import sys

import pytest
import salt.utils.files
import salt.utils.path
import salt.utils.platform
import salt.utils.versions
import salt.utils.win_dacl
import salt.utils.win_functions
import salt.utils.win_runas
from salt.exceptions import CommandExecutionError
from salt.ext import six
from salt.modules.virtualenv_mod import KNOWN_BINARY_NAMES
from tests.support.case import ModuleCase
from tests.support.helpers import (
    destructiveTest,
    patched_environ,
    requires_system_grains,
    skip_if_not_root,
    slowTest,
    with_system_user,
    with_tempdir,
)
from tests.support.mixins import SaltReturnAssertsMixin
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import skipIf

try:
    import pwd

    HAS_PWD = True
except ImportError:
    HAS_PWD = False


class VirtualEnv(object):
    def __init__(self, test, venv_dir):
        self.venv_dir = venv_dir
        self.test = test
        self.test.addCleanup(shutil.rmtree, self.venv_dir, ignore_errors=True)

    def __enter__(self):
        ret = self.test._create_virtualenv(self.venv_dir)
        self.test.assertEqual(
            ret["retcode"],
            0,
            msg="Expected 'retcode' key did not match. Full return dictionary:\n{}".format(
                pprint.pformat(ret)
            ),
        )

    def __exit__(self, *args):
        pass


@skipIf(
    salt.utils.path.which_bin(KNOWN_BINARY_NAMES) is None, "virtualenv not installed"
)
@pytest.mark.windows_whitelisted
class PipStateTest(ModuleCase, SaltReturnAssertsMixin):
    def _win_user_where(self, username, password, program):
        cmd = "cmd.exe /c where {}".format(program)
        ret = salt.utils.win_runas.runas(cmd, username, password)
        assert ret["retcode"] == 0, "{} returned {}".format(cmd, ret["retcode"])
        return ret["stdout"].strip().split("\n")[-1].strip()

    def _create_virtualenv(self, path, **kwargs):
        """
        The reason why the virtualenv creation is proxied by this function is mostly
        because under windows, we can't seem to properly create a virtualenv off of
        another virtualenv(we can on linux) and also because, we really don't want to
        test virtualenv creation off of another virtualenv, we want a virtualenv created
        from the original python.
        Also, one windows, we must also point to the virtualenv binary outside the existing
        virtualenv because it will fail otherwise
        """
        self.addCleanup(shutil.rmtree, path, ignore_errors=True)
        if "python" not in kwargs:
            try:
                if salt.utils.platform.is_windows():
                    python = os.path.join(
                        sys.real_prefix, os.path.basename(sys.executable)
                    )
                else:
                    python_binary_names = [
                        "python{}.{}".format(*sys.version_info),
                        "python{}".format(*sys.version_info),
                        "python",
                    ]
                    for binary_name in python_binary_names:
                        python = os.path.join(sys.real_prefix, "bin", binary_name)
                        if os.path.exists(python):
                            break
                    else:
                        self.fail(
                            "Couldn't find a python binary name under '{}' matching: {}".format(
                                os.path.join(sys.real_prefix, "bin"),
                                python_binary_names,
                            )
                        )
                # We're running off a virtualenv, and we don't want to create a virtualenv off of
                # a virtualenv, let's point to the actual python that created the virtualenv
                kwargs["python"] = python
            except AttributeError:
                # We're running off of the system python
                pass
        return self.run_function("virtualenv.create", [path], **kwargs)

    @slowTest
    def test_pip_installed_removed(self):
        """
        Tests installed and removed states
        """
        name = "pudb"
        if name in self.run_function("pip.list"):
            self.skipTest(
                "{0} is already installed, uninstall to run this test".format(name)
            )
        ret = self.run_state("pip.installed", name=name)
        self.assertSaltTrueReturn(ret)
        ret = self.run_state("pip.removed", name=name)
        self.assertSaltTrueReturn(ret)

    @slowTest
    def test_pip_installed_removed_venv(self):
        venv_dir = os.path.join(RUNTIME_VARS.TMP, "pip_installed_removed")
        with VirtualEnv(self, venv_dir):
            name = "pudb"
            ret = self.run_state("pip.installed", name=name, bin_env=venv_dir)
            self.assertSaltTrueReturn(ret)
            ret = self.run_state("pip.removed", name=name, bin_env=venv_dir)
            self.assertSaltTrueReturn(ret)

    @slowTest
    def test_pip_installed_errors(self):
        venv_dir = os.path.join(RUNTIME_VARS.TMP, "pip-installed-errors")
        self.addCleanup(shutil.rmtree, venv_dir, ignore_errors=True)
        # Since we don't have the virtualenv created, pip.installed will
        # throw an error.
        # Example error strings:
        #  * "Error installing 'pep8': /tmp/pip-installed-errors: not found"
        #  * "Error installing 'pep8': /bin/sh: 1: /tmp/pip-installed-errors: not found"
        #  * "Error installing 'pep8': /bin/bash: /tmp/pip-installed-errors: No such file or directory"
        with patched_environ(SHELL="/bin/sh"):
            ret = self.run_function("state.sls", mods="pip-installed-errors")
            self.assertSaltFalseReturn(ret)
            self.assertSaltCommentRegexpMatches(ret, "Error installing 'pep8':")

            # We now create the missing virtualenv
            ret = self.run_function("virtualenv.create", [venv_dir])
            self.assertEqual(ret["retcode"], 0)

            # The state should not have any issues running now
            ret = self.run_function("state.sls", mods="pip-installed-errors")
            self.assertSaltTrueReturn(ret)

    @skipIf(six.PY3, "Issue is specific to carbon module, which is PY2-only")
    @skipIf(salt.utils.platform.is_windows(), "Carbon does not install in Windows")
    @requires_system_grains
    def test_pip_installed_weird_install(self, grains=None):
        # First, check to see if this is running on CentOS 5 or MacOS.
        # If so, skip this test.
        if grains["os"] in ("CentOS",) and grains["osrelease_info"][0] in (5,):
            self.skipTest("This test does not run reliably on CentOS 5")
        if grains["os"] in ("MacOS",):
            self.skipTest("This test does not run reliably on MacOS")

        ographite = "/opt/graphite"
        if os.path.isdir(ographite):
            self.skipTest(
                "You already have '{0}'. This test would overwrite this "
                "directory".format(ographite)
            )
        try:
            os.makedirs(ographite)
        except OSError as err:
            if err.errno == errno.EACCES:
                # Permission denied
                self.skipTest(
                    "You don't have the required permissions to run this test"
                )
        finally:
            if os.path.isdir(ographite):
                shutil.rmtree(ographite, ignore_errors=True)

        venv_dir = os.path.join(RUNTIME_VARS.TMP, "pip-installed-weird-install")
        try:
            # We may be able to remove this, I had to add it because the custom
            # modules from the test suite weren't available in the jinja
            # context when running the call to state.sls that comes after.
            self.run_function("saltutil.sync_modules")
            # Since we don't have the virtualenv created, pip.installed will
            # throw an error.
            ret = self.run_function("state.sls", mods="pip-installed-weird-install")
            self.assertSaltTrueReturn(ret)

            # We cannot use assertInSaltComment here because we need to skip
            # some of the state return parts
            for key in six.iterkeys(ret):
                self.assertTrue(ret[key]["result"])
                if ret[key]["name"] != "carbon < 1.1":
                    continue
                self.assertEqual(
                    ret[key]["comment"],
                    "There was no error installing package 'carbon < 1.1' "
                    "although it does not show when calling 'pip.freeze'.",
                )
                break
            else:
                raise Exception("Expected state did not run")
        finally:
            if os.path.isdir(ographite):
                shutil.rmtree(ographite, ignore_errors=True)

    @slowTest
    def test_issue_2028_pip_installed_state(self):
        ret = self.run_function("state.sls", mods="issue-2028-pip-installed")

        venv_dir = os.path.join(RUNTIME_VARS.TMP, "issue-2028-pip-installed")
        self.addCleanup(shutil.rmtree, venv_dir, ignore_errors=True)

        pep8_bin = os.path.join(venv_dir, "bin", "pep8")
        if salt.utils.platform.is_windows():
            pep8_bin = os.path.join(venv_dir, "Scripts", "pep8.exe")

        self.assertSaltTrueReturn(ret)
        self.assertTrue(os.path.isfile(pep8_bin))

    @slowTest
    def test_issue_2087_missing_pip(self):
        venv_dir = os.path.join(RUNTIME_VARS.TMP, "issue-2087-missing-pip")

        # Let's create the testing virtualenv
        ret = self._create_virtualenv(venv_dir)
        self.assertEqual(
            ret["retcode"],
            0,
            msg="Expected 'retcode' key did not match. Full return dictionary:\n{}".format(
                pprint.pformat(ret)
            ),
        )

        # Let's remove the pip binary
        pip_bin = os.path.join(venv_dir, "bin", "pip")
        site_dir = self.run_function(
            "virtualenv.get_distribution_path", [venv_dir, "pip"]
        )
        if salt.utils.platform.is_windows():
            pip_bin = os.path.join(venv_dir, "Scripts", "pip.exe")
            site_dir = os.path.join(venv_dir, "lib", "site-packages")
        if not os.path.isfile(pip_bin):
            self.skipTest("Failed to find the pip binary to the test virtualenv")
        os.remove(pip_bin)

        # Also remove the pip dir from site-packages
        # This is needed now that we're using python -m pip instead of the
        # pip binary directly. python -m pip will still work even if the
        # pip binary is missing
        shutil.rmtree(os.path.join(site_dir, "pip"))

        # Let's run the state which should fail because pip is missing
        ret = self.run_function("state.sls", mods="issue-2087-missing-pip")
        self.assertSaltFalseReturn(ret)
        self.assertInSaltComment(
            "Error installing 'pep8': Could not find a `pip` binary", ret
        )

    def test_issue_5940_multiple_pip_mirrors(self):
        """
        Test multiple pip mirrors.  This test only works with pip < 7.0.0
        """
        ret = self.run_function("state.sls", mods="issue-5940-multiple-pip-mirrors")

        venv_dir = os.path.join(RUNTIME_VARS.TMP, "5940-multiple-pip-mirrors")
        self.addCleanup(shutil.rmtree, venv_dir, ignore_errors=True)

        try:
            self.assertSaltTrueReturn(ret)
            self.assertTrue(os.path.isfile(os.path.join(venv_dir, "bin", "pep8")))
        except (AssertionError, CommandExecutionError):
            pip_version = self.run_function("pip.version", [venv_dir])
            if salt.utils.versions.compare(ver1=pip_version, oper=">=", ver2="7.0.0"):
                self.skipTest(
                    "the --mirrors arg has been deprecated and removed in pip==7.0.0"
                )

    @destructiveTest
    @skip_if_not_root
    @with_system_user(
        "issue-6912", on_existing="delete", delete=True, password="PassWord1!"
    )
    @with_tempdir()
    @slowTest
    def test_issue_6912_wrong_owner(self, temp_dir, username):
        # Setup virtual environment directory to be used throughout the test
        venv_dir = os.path.join(temp_dir, "6912-wrong-owner")
        venv_kwargs = {}

        # The virtual environment needs to be in a location that is accessible
        # by both the user running the test and the runas user
        if salt.utils.platform.is_windows():
            salt.utils.win_dacl.set_permissions(temp_dir, username, "full_control")
            # Make sure we're calling a virtualenv and python
            # program that the user has access too.
            venv_kwargs["venv_bin"] = self._win_user_where(
                username, "PassWord1!", "virtualenv",
            )
            venv_kwargs["python"] = self._win_user_where(
                username, "PassWord1!", "python",
            )
        else:
            uid = self.run_function("file.user_to_uid", [username])
            os.chown(temp_dir, uid, -1)

        # Create the virtual environment
        venv_create = self._create_virtualenv(
            venv_dir, user=username, password="PassWord1!", **venv_kwargs
        )
        if venv_create.get("retcode", 1) > 0:
            self.skipTest(
                "Failed to create testcase virtual environment: {0}"
                "".format(venv_create)
            )

        # pip install passing the package name in `name`
        ret = self.run_state(
            "pip.installed",
            name="pep8",
            user=username,
            bin_env=venv_dir,
            password="PassWord1!",
        )
        self.assertSaltTrueReturn(ret)

        if HAS_PWD:
            uid = pwd.getpwnam(username).pw_uid
        for globmatch in (
            os.path.join(venv_dir, "**", "pep8*"),
            os.path.join(venv_dir, "*", "**", "pep8*"),
            os.path.join(venv_dir, "*", "*", "**", "pep8*"),
        ):
            for path in glob.glob(globmatch):
                if HAS_PWD:
                    self.assertEqual(uid, os.stat(path).st_uid)
                elif salt.utils.platform.is_windows():
                    self.assertEqual(salt.utils.win_dacl.get_owner(path), username)

    @destructiveTest
    @skip_if_not_root
    @skipIf(salt.utils.platform.is_darwin(), "Test is flaky on macosx")
    @with_system_user(
        "issue-6912", on_existing="delete", delete=True, password="PassWord1!"
    )
    @with_tempdir()
    @slowTest
    def test_issue_6912_wrong_owner_requirements_file(self, temp_dir, username):
        # Setup virtual environment directory to be used throughout the test
        venv_dir = os.path.join(temp_dir, "6912-wrong-owner")
        venv_kwargs = {}

        # The virtual environment needs to be in a location that is accessible
        # by both the user running the test and the runas user
        if salt.utils.platform.is_windows():
            salt.utils.win_dacl.set_permissions(temp_dir, username, "full_control")
            # Make sure we're calling a virtualenv and python
            # program that the user has access too.
            venv_kwargs["venv_bin"] = self._win_user_where(
                username, "PassWord1!", "virtualenv",
            )
            venv_kwargs["python"] = self._win_user_where(
                username, "PassWord1!", "python",
            )
        else:
            uid = self.run_function("file.user_to_uid", [username])
            os.chown(temp_dir, uid, -1)

        # Create the virtual environment again as it should have been removed
        venv_create = self._create_virtualenv(
            venv_dir, user=username, password="PassWord1!", **venv_kwargs
        )
        if venv_create.get("retcode", 1) > 0:
            self.skipTest(
                "failed to create testcase virtual environment: {0}"
                "".format(venv_create)
            )

        # pip install using a requirements file
        req_filename = os.path.join(
            RUNTIME_VARS.TMP_STATE_TREE, "issue-6912-requirements.txt"
        )
        with salt.utils.files.fopen(req_filename, "wb") as reqf:
            reqf.write(b"pep8\n")

        ret = self.run_state(
            "pip.installed",
            name="",
            user=username,
            bin_env=venv_dir,
            requirements="salt://issue-6912-requirements.txt",
            password="PassWord1!",
        )
        self.assertSaltTrueReturn(ret)

        if HAS_PWD:
            uid = pwd.getpwnam(username).pw_uid
        for globmatch in (
            os.path.join(venv_dir, "**", "pep8*"),
            os.path.join(venv_dir, "*", "**", "pep8*"),
            os.path.join(venv_dir, "*", "*", "**", "pep8*"),
        ):
            for path in glob.glob(globmatch):
                if HAS_PWD:
                    self.assertEqual(uid, os.stat(path).st_uid)
                elif salt.utils.platform.is_windows():
                    self.assertEqual(salt.utils.win_dacl.get_owner(path), username)

    @destructiveTest
    @slowTest
    def test_issue_6833_pip_upgrade_pip(self):
        # Create the testing virtualenv
        if sys.platform == "win32":
            # To keeps the path short, we'll create this directory on the root
            # of the system drive. Otherwise the path is too long and the pip
            # upgrade will fail. Also, I don't know why salt.utils.platform
            # doesn't work in this function, that's why I used sys.platform
            # Need to use os.sep.join here instead of os.path.join because of
            # the colon in SystemDrive
            venv_dir = os.sep.join(
                [os.environ["SystemDrive"], "tmp-6833-pip-upgrade-pip"]
            )
        else:
            venv_dir = os.path.join(RUNTIME_VARS.TMP, "6833-pip-upgrade-pip")
        ret = self._create_virtualenv(venv_dir)

        self.assertEqual(
            ret["retcode"],
            0,
            msg="Expected 'retcode' key did not match. Full return dictionary:\n{}".format(
                pprint.pformat(ret)
            ),
        )

        if not (
            "New python executable" in ret["stdout"]
            or "created virtual environment" in ret["stdout"]
        ):
            assert (
                False
            ), "Expected STDOUT did not match. Full return dictionary:\n{}".format(
                pprint.pformat(ret)
            )

        # Let's install a fixed version pip over whatever pip was
        # previously installed
        ret = self.run_function(
            "pip.install", ["pip==9.0.1"], upgrade=True, bin_env=venv_dir
        )

        if not isinstance(ret, dict):
            self.fail(
                "The 'pip.install' command did not return the excepted dictionary. Output:\n{}".format(
                    ret
                )
            )

        self.assertEqual(ret["retcode"], 0)
        self.assertIn("Successfully installed pip", ret["stdout"])

        # Let's make sure we have pip 9.0.1 installed
        self.assertEqual(
            self.run_function("pip.list", ["pip"], bin_env=venv_dir), {"pip": "9.0.1"}
        )

        # Now the actual pip upgrade pip test
        ret = self.run_state(
            "pip.installed", name="pip==20.0.1", upgrade=True, bin_env=venv_dir
        )

        if not isinstance(ret, dict):
            self.fail(
                "The 'pip.install' command did not return the excepted dictionary. Output:\n{}".format(
                    ret
                )
            )

        self.assertSaltTrueReturn(ret)
        self.assertSaltStateChangesEqual(ret, {"pip==20.0.1": "Installed"})

    @slowTest
    def test_pip_installed_specific_env(self):
        # Create the testing virtualenv
        venv_dir = os.path.join(RUNTIME_VARS.TMP, "pip-installed-specific-env")

        # Let's write a requirements file
        requirements_file = os.path.join(
            RUNTIME_VARS.TMP_PRODENV_STATE_TREE, "prod-env-requirements.txt"
        )
        with salt.utils.files.fopen(requirements_file, "wb") as reqf:
            reqf.write(b"pep8\n")

        try:
            self._create_virtualenv(venv_dir)

            # The requirements file should not be found the base environment
            ret = self.run_state(
                "pip.installed",
                name="",
                bin_env=venv_dir,
                requirements="salt://prod-env-requirements.txt",
            )
            self.assertSaltFalseReturn(ret)
            self.assertInSaltComment(
                "'salt://prod-env-requirements.txt' not found", ret
            )

            # The requirements file must be found in the prod environment
            ret = self.run_state(
                "pip.installed",
                name="",
                bin_env=venv_dir,
                saltenv="prod",
                requirements="salt://prod-env-requirements.txt",
            )
            self.assertSaltTrueReturn(ret)
            self.assertInSaltComment(
                "Successfully processed requirements file "
                "salt://prod-env-requirements.txt",
                ret,
            )

            # We're using the base environment but we're passing the prod
            # environment as an url arg to salt://
            ret = self.run_state(
                "pip.installed",
                name="",
                bin_env=venv_dir,
                requirements="salt://prod-env-requirements.txt?saltenv=prod",
            )
            self.assertSaltTrueReturn(ret)
            self.assertInSaltComment("Requirements were already installed.", ret)
        finally:
            if os.path.isfile(requirements_file):
                os.unlink(requirements_file)

    @skipIf(
        salt.utils.platform.is_darwin() and six.PY2, "This test hangs on OS X on Py2"
    )
    @slowTest
    def test_22359_pip_installed_unless_does_not_trigger_warnings(self):
        # This test case should be moved to a format_call unit test specific to
        # the state internal keywords
        venv_dir = os.path.join(RUNTIME_VARS.TMP, "pip-installed-unless")
        venv_create = self._create_virtualenv(venv_dir)
        if venv_create.get("retcode", 1) > 0:
            self.skipTest(
                "Failed to create testcase virtual environment: {0}".format(venv_create)
            )

        false_cmd = RUNTIME_VARS.SHELL_FALSE_PATH
        if salt.utils.platform.is_windows():
            false_cmd = "exit 1 >nul"
        try:
            ret = self.run_state(
                "pip.installed",
                name="pep8",
                bin_env=venv_dir,
                unless=false_cmd,
                timeout=600,
            )
            self.assertSaltTrueReturn(ret)
            self.assertNotIn("warnings", next(six.itervalues(ret)))
        finally:
            if os.path.isdir(venv_dir):
                shutil.rmtree(venv_dir, ignore_errors=True)

    @skipIf(
        sys.version_info[:2] >= (3, 6),
        "Old version of virtualenv too old for python3.6",
    )
    @skipIf(salt.utils.platform.is_windows(), "Carbon does not install in Windows")
    @slowTest
    def test_46127_pip_env_vars(self):
        """
        Test that checks if env_vars passed to pip.installed are also passed
        to pip.freeze while checking for existing installations
        """
        # This issue is most easily checked while installing carbon
        # Much of the code here comes from the test_weird_install function above
        ographite = "/opt/graphite"
        if os.path.isdir(ographite):
            self.skipTest(
                "You already have '{0}'. This test would overwrite this "
                "directory".format(ographite)
            )
        try:
            os.makedirs(ographite)
        except OSError as err:
            if err.errno == errno.EACCES:
                # Permission denied
                self.skipTest(
                    "You don't have the required permissions to run this test"
                )
        finally:
            if os.path.isdir(ographite):
                shutil.rmtree(ographite, ignore_errors=True)

        venv_dir = os.path.join(RUNTIME_VARS.TMP, "issue-46127-pip-env-vars")
        try:
            # We may be able to remove this, I had to add it because the custom
            # modules from the test suite weren't available in the jinja
            # context when running the call to state.sls that comes after.
            self.run_function("saltutil.sync_modules")
            # Since we don't have the virtualenv created, pip.installed will
            # throw an error.
            ret = self.run_function("state.sls", mods="issue-46127-pip-env-vars")
            self.assertSaltTrueReturn(ret)
            for key in six.iterkeys(ret):
                self.assertTrue(ret[key]["result"])
                if ret[key]["name"] != "carbon < 1.3":
                    continue
                self.assertEqual(
                    ret[key]["comment"], "All packages were successfully installed"
                )
                break
            else:
                raise Exception("Expected state did not run")
            # Run the state again. Now the already installed message should
            # appear
            ret = self.run_function("state.sls", mods="issue-46127-pip-env-vars")
            self.assertSaltTrueReturn(ret)
            # We cannot use assertInSaltComment here because we need to skip
            # some of the state return parts
            for key in six.iterkeys(ret):
                self.assertTrue(ret[key]["result"])
                # As we are re-running the formula, some states will not be run
                # and "name" may or may not be present, so we use .get() pattern
                if ret[key].get("name", "") != "carbon < 1.3":
                    continue
                self.assertEqual(
                    ret[key]["comment"], ("All packages were successfully installed")
                )
                break
            else:
                raise Exception("Expected state did not run")
        finally:
            if os.path.isdir(ographite):
                shutil.rmtree(ographite, ignore_errors=True)
            if os.path.isdir(venv_dir):
                shutil.rmtree(venv_dir)


@pytest.mark.windows_whitelisted
class PipStateInRequisiteTest(ModuleCase, SaltReturnAssertsMixin):
    @with_tempdir()
    @slowTest
    def test_issue_54755(self, tmpdir):
        """
        Verify github issue 54755 is resolved. This only fails when there is no
        pip module in the python environment. Since the test suite normally has
        a pip module this test will pass and is here for posterity. See also

        unit.states.test_pip_state.PipStateUtilsTest.test_pip_purge_method_with_pip

         and

        unit.states.test_pip_state.PipStateUtilsTest.test_pip_purge_method_without_pip

        Which also validate this issue and will pass/fail regardless of whether
        or not pip is installed.
        """
        file_path = os.path.join(tmpdir, "issue-54755")
        ret = self.run_function(
            "state.sls", mods="issue-54755", pillar={"file_path": file_path}
        )
        key = "file_|-issue-54755_|-{}_|-managed".format(file_path)
        assert key in ret
        assert ret[key]["result"] is True
        with salt.utils.files.fopen(file_path, "r") as fp:
            assert fp.read().strip() == "issue-54755"



# -*- coding: utf-8 -*-
"""
    integration.cli_test
    ~~~~~~~~~~~~~~~~~~~~

    CLI related unit testing

    :codeauthor: Pedro Algarvio (pedro@algarvio.me)
"""
from __future__ import absolute_import, print_function

# pylint: disable=import-error
from salt.ext.six.moves import range  # pylint: disable=redefined-builtin
from tests.support.case import ShellCase
from tests.support.helpers import slowTest
from tests.support.mixins import ShellCaseCommonTestsMixin
from tests.support.unit import skipIf

try:
    import libcloud  # pylint: disable=unused-import

    HAS_LIBCLOUD = True
except ImportError:
    HAS_LIBCLOUD = False
# pylint: enable=import-error


@skipIf(HAS_LIBCLOUD is False, "salt-cloud requires >= libcloud 0.11.4")
class SaltCloudCliTest(ShellCase, ShellCaseCommonTestsMixin):

    _call_binary_ = "salt-cloud"

    @slowTest
    def test_function_arguments(self):
        self.assertIn(
            "error: --function expects two arguments: " "<function-name> <provider>",
            "\n".join(self.run_cloud("--function show_image -h", catch_stderr=True)[1]),
        )

    @slowTest
    def test_list_providers_accepts_no_arguments(self):
        self.assertIn(
            "error: '--list-providers' does not accept any " "arguments",
            "\n".join(self.run_cloud("--list-providers ec2", catch_stderr=True)[1]),
        )

    @slowTest
    def test_mutually_exclusive_query_options(self):
        test_options = ["--query", "--full-query", "--select-query", "--list-providers"]
        while True:
            for idx in range(1, len(test_options)):
                self.assertIn(
                    "error: The options {0}/{1} are mutually "
                    "exclusive. Please only choose one of them".format(
                        test_options[0], test_options[idx]
                    ),
                    "\n".join(
                        self.run_cloud(
                            "{0} {1}".format(test_options[0], test_options[idx]),
                            catch_stderr=True,
                        )[1]
                    ),
                )
            # Remove the first option from the list
            test_options.pop(0)
            if len(test_options) <= 1:
                # Only one left? Stop iterating
                break

    @slowTest
    def test_mutually_exclusive_list_options(self):
        test_options = ["--list-locations", "--list-images", "--list-sizes"]
        while True:
            for idx in range(1, len(test_options)):
                output = self.run_cloud(
                    "{0} ec2 {1} ec2".format(test_options[0], test_options[idx]),
                    catch_stderr=True,
                )
                try:
                    self.assertIn(
                        "error: The options {0}/{1} are mutually "
                        "exclusive. Please only choose one of them".format(
                            test_options[0], test_options[idx]
                        ),
                        "\n".join(output[1]),
                    )
                except AssertionError:
                    print(output)
                    raise
            # Remove the first option from the list
            test_options.pop(0)
            if len(test_options) <= 1:
                # Only one left? Stop iterating
                break



# -*- coding: utf-8 -*-
"""
    :codeauthor: :email:`Gareth J. Greenaway <gareth@saltstack.com>`
    :codeauthor: :email:`David Murphy <dmurphy@saltstack.com>`
"""

from __future__ import absolute_import, print_function, unicode_literals

import datetime
import os
import shutil
import time

import salt.modules.gpg as gpg
import salt.utils.files
import salt.utils.platform
from tests.support.helpers import destructiveTest, slowTest
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import TestCase, skipIf

GPG_TEST_KEY_PASSPHRASE = "testkeypassphrase"
GPG_TEST_KEY_ID = "7416F045"
GPG_TEST_PUB_KEY = """-----BEGIN PGP PUBLIC KEY BLOCK-----
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=yF8M
-----END PGP PUBLIC KEY BLOCK-----
"""

GPG_TEST_PRIV_KEY = """-----BEGIN PGP PRIVATE KEY BLOCK-----


OZV2Hg+93dg3Wi6g/JW4OuTKWKuHRqpRB1J4i4lO
=WRTN
-----END PGP PRIVATE KEY BLOCK-----
"""

try:
    import gnupg  # pylint: disable=import-error,unused-import

    HAS_GPG = True
except ImportError:
    HAS_GPG = False


@destructiveTest
@skipIf(not salt.utils.platform.is_linux(), "These tests can only be run on linux")
class GpgTestCase(TestCase, LoaderModuleMockMixin):
    """
    Validate the gpg module
    """

    def setup_loader_modules(self):
        return {gpg: {"__salt__": {}}}

    @skipIf(not HAS_GPG, "GPG Module Unavailable")
    def setUp(self):
        super(GpgTestCase, self).setUp()
        self.gpghome = os.path.join(RUNTIME_VARS.TMP, "gpghome")
        if not os.path.isdir(self.gpghome):
            # left behind... Don't fail because of this!
            os.makedirs(self.gpghome)
        self.gpgfile_pub = os.path.join(RUNTIME_VARS.TMP, "gpgfile.pub")
        with salt.utils.files.fopen(self.gpgfile_pub, "wb") as fp:
            fp.write(salt.utils.stringutils.to_bytes(GPG_TEST_PUB_KEY))
        self.gpgfile_priv = os.path.join(RUNTIME_VARS.TMP, "gpgfile.priv")
        with salt.utils.files.fopen(self.gpgfile_priv, "wb") as fp:
            fp.write(salt.utils.stringutils.to_bytes(GPG_TEST_PRIV_KEY))
        self.user = "salt"

    @skipIf(not HAS_GPG, "GPG Module Unavailable")
    def tearDown(self):
        if os.path.isfile(self.gpgfile_pub):
            os.remove(self.gpgfile_pub)
        shutil.rmtree(self.gpghome, ignore_errors=True)
        super(GpgTestCase, self).tearDown()

    @skipIf(not HAS_GPG, "GPG Module Unavailable")
    def test_list_keys(self):
        """
        Test gpg.list_keys
        """

        _user_mock = {
            "shell": "/bin/bash",
            "workphone": "",
            "uid": 0,
            "passwd": "x",
            "roomnumber": "",
            "gid": 0,
            "groups": ["root"],
            "home": "/root",
            "fullname": "root",
            "homephone": "",
            "name": "root",
        }

        _list_result = [
            {
                "dummy": "",
                "keyid": "xxxxxxxxxxxxxxxx",
                "expires": "2011188692",
                "sigs": [],
                "subkeys": [
                    [
                        "xxxxxxxxxxxxxxxx",
                        "e",
                        "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                    ]
                ],
                "length": "4096",
                "ownertrust": "-",
                "sig": "",
                "algo": "1",
                "fingerprint": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                "date": "1506612692",
                "trust": "-",
                "type": "pub",
                "uids": ["GPG Person <person@example.com>"],
            }
        ]

        _expected_result = [
            {
                "keyid": "xxxxxxxxxxxxxxxx",
                "uids": ["GPG Person <person@example.com>"],
                "created": "2017-09-28",
                "expires": "2033-09-24",
                "keyLength": "4096",
                "ownerTrust": "Unknown",
                "fingerprint": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                "trust": "Unknown",
            }
        ]

        mock_opt = MagicMock(return_value="root")
        with patch.dict(
            gpg.__salt__, {"user.info": MagicMock(return_value=_user_mock)}
        ):
            with patch.dict(gpg.__salt__, {"config.option": mock_opt}):
                with patch.object(gpg, "_list_keys", return_value=_list_result):
                    self.assertEqual(gpg.list_keys(), _expected_result)

    @skipIf(not HAS_GPG, "GPG Module Unavailable")
    def test_get_key(self):
        """
        Test gpg.get_key
        """

        _user_mock = {
            "shell": "/bin/bash",
            "workphone": "",
            "uid": 0,
            "passwd": "x",
            "roomnumber": "",
            "gid": 0,
            "groups": ["root"],
            "home": "/root",
            "fullname": "root",
            "homephone": "",
            "name": "root",
        }

        _list_result = [
            {
                "dummy": "",
                "keyid": "xxxxxxxxxxxxxxxx",
                "expires": "2011188692",
                "sigs": [],
                "subkeys": [
                    [
                        "xxxxxxxxxxxxxxxx",
                        "e",
                        "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                    ]
                ],
                "length": "4096",
                "ownertrust": "-",
                "sig": "",
                "algo": "1",
                "fingerprint": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                "date": "1506612692",
                "trust": "-",
                "type": "pub",
                "uids": ["GPG Person <person@example.com>"],
            }
        ]

        _expected_result = {
            "fingerprint": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            "keyid": "xxxxxxxxxxxxxxxx",
            "uids": ["GPG Person <person@example.com>"],
            "created": "2017-09-28",
            "trust": "Unknown",
            "ownerTrust": "Unknown",
            "expires": "2033-09-24",
            "keyLength": "4096",
        }

        mock_opt = MagicMock(return_value="root")
        with patch.dict(
            gpg.__salt__, {"user.info": MagicMock(return_value=_user_mock)}
        ):
            with patch.dict(gpg.__salt__, {"config.option": mock_opt}):
                with patch.object(gpg, "_list_keys", return_value=_list_result):
                    ret = gpg.get_key("xxxxxxxxxxxxxxxx")
                    self.assertEqual(ret, _expected_result)

    @destructiveTest  # Need to run as root!?
    @skipIf(not HAS_GPG, "GPG Module Unavailable")
    def test_delete_key(self):
        """
        Test gpg.delete_key
        """

        _user_mock = {
            "shell": "/bin/bash",
            "workphone": "",
            "uid": 0,
            "passwd": "x",
            "roomnumber": "",
            "gid": 0,
            "groups": ["root"],
            "home": self.gpghome,
            "fullname": "root",
            "homephone": "",
            "name": "root",
        }

        _list_result = [
            {
                "dummy": "",
                "keyid": "xxxxxxxxxxxxxxxx",
                "expires": "2011188692",
                "sigs": [],
                "subkeys": [
                    [
                        "xxxxxxxxxxxxxxxx",
                        "e",
                        "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                    ]
                ],
                "length": "4096",
                "ownertrust": "-",
                "sig": "",
                "algo": "1",
                "fingerprint": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                "date": "1506612692",
                "trust": "-",
                "type": "pub",
                "uids": ["GPG Person <person@example.com>"],
            }
        ]

        _expected_result = {
            "res": True,
            "message": "Secret key for xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx deleted\nPublic key for xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx deleted",
        }

        mock_opt = MagicMock(return_value="root")
        with patch.dict(
            gpg.__salt__, {"user.info": MagicMock(return_value=_user_mock)}
        ):
            with patch.dict(gpg.__salt__, {"config.option": mock_opt}):
                with patch.object(gpg, "_list_keys", return_value=_list_result):
                    with patch(
                        "salt.modules.gpg.gnupg.GPG.delete_keys",
                        MagicMock(return_value="ok"),
                    ):
                        ret = gpg.delete_key("xxxxxxxxxxxxxxxx", delete_secret=True)
                        self.assertEqual(ret, _expected_result)

    @skipIf(not HAS_GPG, "GPG Module Unavailable")
    def test_search_keys(self):
        """
        Test gpg.search_keys
        """

        _user_mock = {
            "shell": "/bin/bash",
            "workphone": "",
            "uid": 0,
            "passwd": "x",
            "roomnumber": "",
            "gid": 0,
            "groups": ["root"],
            "home": self.gpghome,
            "fullname": "root",
            "homephone": "",
            "name": "root",
        }

        _search_result = [
            {
                "keyid": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                "uids": ["GPG Person <person@example.com>"],
                "expires": "",
                "sigs": [],
                "length": "1024",
                "algo": "17",
                "date": int(time.mktime(datetime.datetime(2004, 11, 13).timetuple())),
                "type": "pub",
            }
        ]

        _expected_result = [
            {
                "uids": ["GPG Person <person@example.com>"],
                "created": "2004-11-13",
                "keyLength": "1024",
                "keyid": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            }
        ]

        mock_opt = MagicMock(return_value="root")
        with patch.dict(
            gpg.__salt__, {"user.info": MagicMock(return_value=_user_mock)}
        ):
            with patch.dict(gpg.__salt__, {"config.option": mock_opt}):
                with patch.object(gpg, "_search_keys", return_value=_search_result):
                    ret = gpg.search_keys("person@example.com")
                    self.assertEqual(ret, _expected_result)

    @skipIf(not HAS_GPG, "GPG Module Unavailable")
    @slowTest
    def test_gpg_import_pub_key(self):
        config_user = MagicMock(return_value="salt")
        user_info = MagicMock(
            return_value={"name": "salt", "home": self.gpghome, "uid": 1000}
        )
        with patch.dict(gpg.__salt__, {"config.option": config_user}):
            with patch.dict(gpg.__salt__, {"user.info": user_info}):
                ret = gpg.import_key(None, self.gpgfile_pub, "salt", self.gpghome)
                self.assertEqual(ret["res"], True)

    @skipIf(not HAS_GPG, "GPG Module Unavailable")
    @slowTest
    def test_gpg_import_priv_key(self):
        config_user = MagicMock(return_value="salt")
        user_info = MagicMock(
            return_value={"name": "salt", "home": self.gpghome, "uid": 1000}
        )
        with patch.dict(gpg.__salt__, {"config.option": config_user}):
            with patch.dict(gpg.__salt__, {"user.info": user_info}):
                ret = gpg.import_key(None, self.gpgfile_priv, "salt", self.gpghome)
                self.assertEqual(ret["res"], True)

    @skipIf(not HAS_GPG, "GPG Module Unavailable")
    @slowTest
    def test_gpg_sign(self):
        config_user = MagicMock(return_value="salt")
        user_info = MagicMock(
            return_value={"name": "salt", "home": self.gpghome, "uid": 1000}
        )
        pillar_mock = MagicMock(
            return_value={"gpg_passphrase": GPG_TEST_KEY_PASSPHRASE}
        )
        with patch.dict(gpg.__salt__, {"config.option": config_user}):
            with patch.dict(gpg.__salt__, {"user.info": user_info}):
                with patch.dict(gpg.__salt__, {"pillar.get": pillar_mock}):
                    ret = gpg.import_key(None, self.gpgfile_priv, "salt", self.gpghome)
                    self.assertEqual(ret["res"], True)
                    gpg_text_input = "The quick brown fox jumped over the lazy dog"
                    gpg_sign_output = gpg.sign(
                        config_user,
                        GPG_TEST_KEY_ID,
                        gpg_text_input,
                        None,
                        None,
                        True,
                        self.gpghome,
                    )
                    self.assertIsNotNone(gpg_sign_output)



# -*- coding: utf-8 -*-
"""
Module used to access the esx proxy connection methods
"""

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import logging

# Import Salt libs
import salt.utils.platform

log = logging.getLogger(__name__)

__proxyenabled__ = ["esxvm"]
# Define the module's virtual name
__virtualname__ = "esxvm"


def __virtual__():
    """
    Only work on proxy
    """
    if salt.utils.platform.is_proxy():
        return __virtualname__
    return (False, "Must be run on a proxy minion")


def get_details():
    return __proxy__["esxvm.get_details"]()



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.selinux as selinux

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class SelinuxTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.selinux
    """

    def setup_loader_modules(self):
        return {selinux: {}}

    # 'mode' function tests: 1

    def test_mode(self):
        """
        Test to verifies the mode SELinux is running in,
        can be set to enforcing or permissive.
        """
        ret = {
            "name": "unknown",
            "changes": {},
            "result": False,
            "comment": "unknown is not an accepted mode",
        }
        self.assertDictEqual(selinux.mode("unknown"), ret)

        mock_en = MagicMock(return_value="Enforcing")
        mock_pr = MagicMock(side_effect=["Permissive", "Enforcing"])
        with patch.dict(
            selinux.__salt__,
            {
                "selinux.getenforce": mock_en,
                "selinux.getconfig": mock_en,
                "selinux.setenforce": mock_pr,
            },
        ):
            comt = "SELinux is already in Enforcing mode"
            ret = {"name": "Enforcing", "comment": comt, "result": True, "changes": {}}
            self.assertDictEqual(selinux.mode("Enforcing"), ret)

            with patch.dict(selinux.__opts__, {"test": True}):
                comt = "SELinux mode is set to be changed to Permissive"
                ret = {
                    "name": "Permissive",
                    "comment": comt,
                    "result": None,
                    "changes": {"new": "Permissive", "old": "Enforcing"},
                }
                self.assertDictEqual(selinux.mode("Permissive"), ret)

            with patch.dict(selinux.__opts__, {"test": False}):
                comt = "SELinux has been set to Permissive mode"
                ret = {
                    "name": "Permissive",
                    "comment": comt,
                    "result": True,
                    "changes": {"new": "Permissive", "old": "Enforcing"},
                }
                self.assertDictEqual(selinux.mode("Permissive"), ret)

                comt = "Failed to set SELinux to Permissive mode"
                ret.update(
                    {
                        "name": "Permissive",
                        "comment": comt,
                        "result": False,
                        "changes": {},
                    }
                )
                self.assertDictEqual(selinux.mode("Permissive"), ret)

    # 'boolean' function tests: 1

    def test_boolean(self):
        """
        Test to set up an SELinux boolean.
        """
        name = "samba_create_home_dirs"
        value = True
        ret = {"name": name, "changes": {}, "result": False, "comment": ""}

        mock_en = MagicMock(return_value=[])
        with patch.dict(selinux.__salt__, {"selinux.list_sebool": mock_en}):
            comt = "Boolean {0} is not available".format(name)
            ret.update({"comment": comt})
            self.assertDictEqual(selinux.boolean(name, value), ret)

        mock_bools = MagicMock(return_value={name: {"State": "on", "Default": "on"}})
        with patch.dict(selinux.__salt__, {"selinux.list_sebool": mock_bools}):
            comt = "None is not a valid value for the boolean"
            ret.update({"comment": comt})
            self.assertDictEqual(selinux.boolean(name, None), ret)

            comt = "Boolean is in the correct state"
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(selinux.boolean(name, value, True), ret)

            comt = "Boolean is in the correct state"
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(selinux.boolean(name, value), ret)

        mock_bools = MagicMock(return_value={name: {"State": "off", "Default": "on"}})
        mock = MagicMock(side_effect=[True, False])
        with patch.dict(
            selinux.__salt__,
            {"selinux.list_sebool": mock_bools, "selinux.setsebool": mock},
        ):
            with patch.dict(selinux.__opts__, {"test": True}):
                comt = "Boolean samba_create_home_dirs" " is set to be changed to on"
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(selinux.boolean(name, value), ret)

            with patch.dict(selinux.__opts__, {"test": False}):
                comt = "Boolean samba_create_home_dirs has been set to on"
                ret.update({"comment": comt, "result": True})
                ret.update({"changes": {"State": {"old": "off", "new": "on"}}})
                self.assertDictEqual(selinux.boolean(name, value), ret)

                comt = "Failed to set the boolean " "samba_create_home_dirs to on"
                ret.update({"comment": comt, "result": False})
                ret.update({"changes": {}})
                self.assertDictEqual(selinux.boolean(name, value), ret)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Nicole Thomas <nicole@saltstack.com>
"""

# Import Salt Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
from salt.cloud.clouds import linode

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase


class LinodeTestCase(TestCase, LoaderModuleMockMixin):
    """
    Unit TestCase for the salt.cloud.clouds.linode module.
    """

    def setup_loader_modules(self):
        return {linode: {}}

    # _validate_name tests

    def test_validate_name_first_character_invalid(self):
        """
        Tests when name starts with an invalid character.
        """
        # Test when name begins with a hyphen
        self.assertFalse(linode._validate_name("-foo"))

        # Test when name begins with an underscore
        self.assertFalse(linode._validate_name("_foo"))

    def test_validate_name_last_character_invalid(self):
        """
        Tests when name ends with an invalid character.
        """
        # Test when name ends with a hyphen
        self.assertFalse(linode._validate_name("foo-"))

        # Test when name ends with an underscore
        self.assertFalse(linode._validate_name("foo_"))

    def test_validate_name_too_short(self):
        """
        Tests when name has less than three letters.
        """
        # Test when name is an empty string
        self.assertFalse(linode._validate_name(""))

        # Test when name is two letters long
        self.assertFalse(linode._validate_name("ab"))

        # Test when name is three letters long (valid)
        self.assertTrue(linode._validate_name("abc"))

    def test_validate_name_too_long(self):
        """
        Tests when name has more than 48 letters.
        """
        long_name = "1111-2222-3333-4444-5555-6666-7777-8888-9999-111"
        # Test when name is 48 letters long (valid)
        self.assertEqual(len(long_name), 48)
        self.assertTrue(linode._validate_name(long_name))

        # Test when name is more than 48 letters long
        long_name += "1"
        self.assertEqual(len(long_name), 49)
        self.assertFalse(linode._validate_name(long_name))

    def test_validate_name_invalid_characters(self):
        """
        Tests when name contains invalid characters.
        """
        # Test when name contains an invalid character
        self.assertFalse(linode._validate_name("foo;bar"))

        # Test when name contains non-ascii letters
        self.assertFalse(linode._validate_name("fooàààààbar"))

        # Test when name contains spaces
        self.assertFalse(linode._validate_name("foo bar"))

    def test_validate_name_valid_characters(self):
        """
        Tests when name contains valid characters.
        """
        # Test when name contains letters and numbers
        self.assertTrue(linode._validate_name("foo123bar"))

        # Test when name contains hyphens
        self.assertTrue(linode._validate_name("foo-bar"))

        # Test when name contains underscores
        self.assertTrue(linode._validate_name("foo_bar"))

        # Test when name start and end with numbers
        self.assertTrue(linode._validate_name("1foo"))
        self.assertTrue(linode._validate_name("foo0"))



# -*- coding: utf-8 -*-
"""
    :codeauthor: Pedro Algarvio (pedro@algarvio.me)


    tests.integration.shell.syndic
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""

from __future__ import absolute_import

import logging

import psutil
import pytest
import salt.utils.files
import salt.utils.platform
import salt.utils.yaml
from tests.integration.utils import testprogram
from tests.support.case import ShellCase
from tests.support.helpers import slowTest
from tests.support.mixins import ShellCaseCommonTestsMixin
from tests.support.unit import skipIf

try:
    from salt.utils.odict import OrderedDict
except ImportError:
    from collections import OrderedDict

log = logging.getLogger(__name__)


# @pytest.fixture(scope='module', autouse=True)
def session_salt_syndic(request, session_salt_master_of_masters, session_salt_syndic):
    request.session.stats_processes.update(
        OrderedDict(
            (
                (
                    "Salt Syndic Master",
                    psutil.Process(session_salt_master_of_masters.pid),
                ),
                ("Salt Syndic", psutil.Process(session_salt_syndic.pid)),
            )
        ).items()
    )
    yield session_salt_syndic
    request.session.stats_processes.pop("Salt Syndic Master")
    request.session.stats_processes.pop("Salt Syndic")

    # Stop daemons now(they would be stopped at the end of the test run session
    for daemon in (session_salt_syndic, session_salt_master_of_masters):
        try:
            daemon.terminate()
        except Exception as exc:  # pylint: disable=broad-except
            log.warning("Failed to terminate daemon: %s", daemon.__class__.__name__)


@pytest.mark.windows_whitelisted
class SyndicTest(ShellCase, testprogram.TestProgramCase, ShellCaseCommonTestsMixin):
    """
    Test the salt-syndic command
    """

    _call_binary_ = "salt-syndic"

    @skipIf(salt.utils.platform.is_windows(), "Skip on Windows OS")
    @slowTest
    def test_exit_status_unknown_user(self):
        """
        Ensure correct exit status when the syndic is configured to run as an unknown user.

        Skipped on windows because daemonization not supported
        """

        syndic = testprogram.TestDaemonSaltSyndic(
            name="unknown_user",
            config_base={"user": "some_unknown_user_xyz"},
            parent_dir=self._test_dir,
        )
        # Call setup here to ensure config and script exist
        syndic.setup()
        stdout, stderr, status = syndic.run(
            args=["-d"], catch_stderr=True, with_retcode=True,
        )
        try:
            self.assert_exit_status(
                status,
                "EX_NOUSER",
                message="unknown user not on system",
                stdout=stdout,
                stderr=stderr,
            )
        finally:
            # Although the start-up should fail, call shutdown() to set the
            # internal _shutdown flag and avoid the registered atexit calls to
            # cause timeout exceptions and respective traceback
            syndic.shutdown()

    # pylint: disable=invalid-name
    @skipIf(salt.utils.platform.is_windows(), "Skip on Windows OS")
    @slowTest
    def test_exit_status_unknown_argument(self):
        """
        Ensure correct exit status when an unknown argument is passed to salt-syndic.

        Skipped on windows because daemonization not supported
        """

        syndic = testprogram.TestDaemonSaltSyndic(
            name="unknown_argument", parent_dir=self._test_dir,
        )
        # Syndic setup here to ensure config and script exist
        syndic.setup()
        stdout, stderr, status = syndic.run(
            args=["-d", "--unknown-argument"], catch_stderr=True, with_retcode=True,
        )
        try:
            self.assert_exit_status(
                status,
                "EX_USAGE",
                message="unknown argument",
                stdout=stdout,
                stderr=stderr,
            )
        finally:
            # Although the start-up should fail, call shutdown() to set the
            # internal _shutdown flag and avoid the registered atexit calls to
            # cause timeout exceptions and respective traceback
            syndic.shutdown()

    @skipIf(salt.utils.platform.is_windows(), "Skip on Windows OS")
    @slowTest
    def test_exit_status_correct_usage(self):
        """
        Ensure correct exit status when salt-syndic starts correctly.

        Skipped on windows because daemonization not supported
        """

        syndic = testprogram.TestDaemonSaltSyndic(
            name="correct_usage", parent_dir=self._test_dir,
        )
        # Syndic setup here to ensure config and script exist
        syndic.setup()
        stdout, stderr, status = syndic.run(
            args=["-d", "-l", "debug"], catch_stderr=True, with_retcode=True,
        )
        try:
            self.assert_exit_status(
                status, "EX_OK", message="correct usage", stdout=stdout, stderr=stderr
            )
        finally:
            syndic.shutdown(wait_for_orphans=3)



# -*- coding: utf-8 -*-
"""
    :copyright: Copyright 2013-2017 by the SaltStack Team, see AUTHORS for more details.
    :license: Apache 2.0, see LICENSE for more details.


    tests.support.win_installer
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Fetches the binary Windows installer
"""
from __future__ import absolute_import

import hashlib

import requests

PREFIX = "Salt-Minion-"
REPO = "https://repo.saltstack.com/windows"


def latest_installer_name(arch="AMD64", **kwargs):
    """
    Create an installer file name
    """
    return "Salt-Minion-Latest-Py3-{}-Setup.exe".format(arch)


def download_and_verify(fp, name, repo=REPO):
    """
    Download an installer and verify its contents.
    """
    md5 = "{}.md5".format(name)
    url = lambda x: "{}/{}".format(repo, x)
    resp = requests.get(url(md5))
    if resp.status_code != 200:
        raise Exception("Unable to fetch installer md5")
    installer_md5 = resp.text.strip().split()[0].lower()
    resp = requests.get(url(name), stream=True)
    if resp.status_code != 200:
        raise Exception("Unable to fetch installer")
    md5hsh = hashlib.md5()
    for chunk in resp.iter_content(chunk_size=1024):
        md5hsh.update(chunk)
        fp.write(chunk)
    if md5hsh.hexdigest() != installer_md5:
        raise Exception(
            "Installer's hash does not match {} != {}".format(
                md5hsh.hexdigest(), installer_md5
            )
        )



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.modules.locate as locate

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class LocateTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.locate
    """

    def setup_loader_modules(self):
        return {locate: {}}

    # 'version' function tests: 1

    def test_version(self):
        """
        Test if it returns the version of locate
        """
        mock = MagicMock(return_value="mlocate 0.26")
        with patch.dict(locate.__salt__, {"cmd.run": mock}):
            self.assertListEqual(locate.version(), ["mlocate 0.26"])

    # 'stats' function tests: 1

    def test_stats(self):
        """
        Test if it returns statistics about the locate database
        """
        ret = {
            "files": "75,253",
            "directories": "49,252",
            "bytes in file names": "93,214",
            "bytes used to store database": "29,165",
            "database": "/var/lib/mlocate/mlocate.db",
        }

        mock_ret = """Database /var/lib/mlocate/mlocate.db:
        49,252 directories 
        75,253 files 
        93,214 bytes in file names 
        29,165 bytes used to store database"""

        with patch.dict(locate.__salt__, {"cmd.run": MagicMock(return_value=mock_ret)}):
            self.assertDictEqual(locate.stats(), ret)

    # 'updatedb' function tests: 1

    def test_updatedb(self):
        """
        Test if it updates the locate database
        """
        mock = MagicMock(return_value="")
        with patch.dict(locate.__salt__, {"cmd.run": mock}):
            self.assertListEqual(locate.updatedb(), [])

    # 'locate' function tests: 1

    def test_locate(self):
        """
        Test if it performs a file lookup.
        """
        mock = MagicMock(return_value="")
        with patch.dict(locate.__salt__, {"cmd.run": mock}):
            self.assertListEqual(locate.locate("wholename", database="myfile"), [])



# -*- coding: utf-8 -*-
"""
Wrap the saltcheck module to copy files to ssh minion before running tests
"""
# Import Python libs
from __future__ import absolute_import, print_function

import logging
import os
import shutil
import tarfile
import tempfile
from contextlib import closing

import salt.utils.files
import salt.utils.json

# Import salt libs
import salt.utils.url

log = logging.getLogger(__name__)


def update_master_cache(states, saltenv="base"):
    """
    Replace standard saltcheck version with similar logic but replacing cp.cache_dir with
        generating files, tar'ing them up, copying tarball to remote host, extracting tar
        to state cache directory, and cleanup of files
    """
    cache = __opts__["cachedir"]
    state_cache = os.path.join(cache, "files", saltenv)

    # Setup for copying states to gendir
    gendir = tempfile.mkdtemp()
    trans_tar = salt.utils.files.mkstemp()
    if "cp.fileclient_{0}".format(id(__opts__)) not in __context__:
        __context__[
            "cp.fileclient_{0}".format(id(__opts__))
        ] = salt.fileclient.get_file_client(__opts__)

    # generate cp.list_states output and save to gendir
    cp_output = salt.utils.json.dumps(__salt__["cp.list_states"]())
    cp_output_file = os.path.join(gendir, "cp_output.txt")
    with salt.utils.files.fopen(cp_output_file, "w") as fp:
        fp.write(cp_output)

    # cp state directories to gendir
    already_processed = []
    sls_list = salt.utils.args.split_input(states)
    for state_name in sls_list:
        # generate low data for each state and save to gendir
        state_low_file = os.path.join(gendir, state_name + ".low")
        state_low_output = salt.utils.json.dumps(
            __salt__["state.show_low_sls"](state_name)
        )
        with salt.utils.files.fopen(state_low_file, "w") as fp:
            fp.write(state_low_output)

        state_name = state_name.replace(".", os.sep)
        if state_name in already_processed:
            log.debug("Already cached state for %s", state_name)
        else:
            file_copy_file = os.path.join(gendir, state_name + ".copy")
            log.debug("copying %s to %s", state_name, gendir)
            qualified_name = salt.utils.url.create(state_name, saltenv)
            # Duplicate cp.get_dir to gendir
            copy_result = __context__["cp.fileclient_{0}".format(id(__opts__))].get_dir(
                qualified_name, gendir, saltenv
            )
            if copy_result:
                copy_result = [dir.replace(gendir, state_cache) for dir in copy_result]
                copy_result_output = salt.utils.json.dumps(copy_result)
                with salt.utils.files.fopen(file_copy_file, "w") as fp:
                    fp.write(copy_result_output)
                already_processed.append(state_name)
            else:
                # If files were not copied, assume state.file.sls was given and just copy state
                state_name = os.path.dirname(state_name)
                file_copy_file = os.path.join(gendir, state_name + ".copy")
                if state_name in already_processed:
                    log.debug("Already cached state for %s", state_name)
                else:
                    qualified_name = salt.utils.url.create(state_name, saltenv)
                    copy_result = __context__[
                        "cp.fileclient_{0}".format(id(__opts__))
                    ].get_dir(qualified_name, gendir, saltenv)
                    if copy_result:
                        copy_result = [
                            dir.replace(gendir, state_cache) for dir in copy_result
                        ]
                        copy_result_output = salt.utils.json.dumps(copy_result)
                        with salt.utils.files.fopen(file_copy_file, "w") as fp:
                            fp.write(copy_result_output)
                        already_processed.append(state_name)

    # turn gendir into tarball and remove gendir
    try:
        # cwd may not exist if it was removed but salt was run from it
        cwd = os.getcwd()
    except OSError:
        cwd = None
    os.chdir(gendir)
    with closing(tarfile.open(trans_tar, "w:gz")) as tfp:
        for root, dirs, files in salt.utils.path.os_walk(gendir):
            for name in files:
                full = os.path.join(root, name)
                tfp.add(full[len(gendir) :].lstrip(os.sep))
    if cwd:
        os.chdir(cwd)
    shutil.rmtree(gendir)

    # Copy tarfile to ssh host
    single = salt.client.ssh.Single(__opts__, "", **__salt__.kwargs)
    thin_dir = __opts__["thin_dir"]
    ret = single.shell.send(trans_tar, thin_dir)

    # Clean up local tar
    try:
        os.remove(trans_tar)
    except (OSError, IOError):
        pass

    tar_path = os.path.join(thin_dir, os.path.basename(trans_tar))
    # Extract remote tarball to cache directory and remove tar file
    # TODO this could be better handled by a single state/connection due to ssh overhead
    ret = __salt__["file.mkdir"](state_cache)
    ret = __salt__["archive.tar"]("xf", tar_path, dest=state_cache)
    ret = __salt__["file.remove"](tar_path)

    return ret


def run_state_tests(states, saltenv="base", check_all=False):
    """
    Define common functions to activite this wrapping module and tar copy.
    After file copies are finished, run the usual local saltcheck function
    """
    ret = update_master_cache(states, saltenv)
    ret = __salt__["saltcheck.run_state_tests_ssh"](
        states, saltenv=saltenv, check_all=check_all
    )
    return ret


def run_highstate_tests(saltenv="base"):
    """
    Lookup top files for minion, pass results to wrapped run_state_tests for copy and run
    """
    top_states = __salt__["state.show_top"]().get(saltenv)
    state_string = ",".join(top_states)
    ret = run_state_tests(state_string, saltenv)
    return ret



# -*- coding: utf-8 -*-

from __future__ import absolute_import

import time

import pytest
import salt.utils.files
import salt.utils.yaml
from tests.support.case import ShellCase
from tests.support.helpers import dedent, flaky, slowTest
from tests.support.mixins import ShellCaseCommonTestsMixin
from tests.support.unit import skipIf


def minion_in_returns(minion, lines):
    return bool([True for line in lines if line == "{0}:".format(minion)])


@pytest.mark.windows_whitelisted
@pytest.mark.usefixtures("salt_sub_minion")
class MatchTest(ShellCase, ShellCaseCommonTestsMixin):
    """
    Test salt matchers
    """

    RUN_TIMEOUT = 300

    _call_binary_ = "salt"

    @slowTest
    def test_list(self):
        """
        test salt -L matcher
        """
        data = self.run_salt("-L minion test.ping")
        data = "\n".join(data)
        self.assertIn("minion", data)
        self.assertNotIn("sub_minion", data)
        data = self.run_salt("-L minion,sub_minion test.ping")
        data = "\n".join(data)
        self.assertIn("minion", data)
        self.assertIn("sub_minion", data)

    # compound matcher tests: 12

    @slowTest
    def test_compound_min_with_grain(self):
        """
        test salt compound matcher
        """
        data = self.run_salt('-C "min* and G@test_grain:cheese" test.ping')
        assert minion_in_returns("minion", data) is True
        assert minion_in_returns("sub_minion", data) is False

    @slowTest
    def test_compound_and_not_grain(self):
        data = self.run_salt('-C "min* and not G@test_grain:foo" test.ping')
        assert minion_in_returns("minion", data) is True
        assert minion_in_returns("sub_minion", data) is False

    @slowTest
    def test_compound_not_grain(self):
        data = self.run_salt('-C "min* not G@test_grain:foo" test.ping')
        assert minion_in_returns("minion", data) is True
        assert minion_in_returns("sub_minion", data) is False

    @slowTest
    def test_compound_pcre_grain_and_grain(self):
        match = "P@test_grain:^cheese$ and * and G@test_grain:cheese"
        data = self.run_salt('-t 1 -C "{0}" test.ping'.format(match))
        assert minion_in_returns("minion", data) is True
        assert minion_in_returns("sub_minion", data) is False

    @slowTest
    def test_compound_list_and_pcre_minion(self):
        match = "L@sub_minion and E@.*"
        data = self.run_salt('-t 1 -C "{0}" test.ping'.format(match))
        assert minion_in_returns("sub_minion", data) is True
        assert minion_in_returns("minion", data) is False

    @slowTest
    def test_compound_not_sub_minion(self):
        data = self.run_salt('-C "not sub_minion" test.ping')
        assert minion_in_returns("minion", data) is True
        assert minion_in_returns("sub_minion", data) is False

    @slowTest
    def test_compound_all_and_not_grains(self):
        data = self.run_salt('-C "* and ( not G@test_grain:cheese )" test.ping')
        assert minion_in_returns("minion", data) is False
        assert minion_in_returns("sub_minion", data) is True

    @slowTest
    def test_compound_grain_regex(self):
        data = self.run_salt('-C "G%@planets%merc*" test.ping')
        assert minion_in_returns("minion", data) is True
        assert minion_in_returns("sub_minion", data) is False

    @slowTest
    def test_coumpound_pcre_grain_regex(self):
        data = self.run_salt('-C "P%@planets%^(mercury|saturn)$" test.ping')
        assert minion_in_returns("minion", data) is True
        assert minion_in_returns("sub_minion", data) is True

    @skipIf(True, "This test is unreliable. Need to investigate why more deeply.")
    @flaky
    def test_compound_pillar(self):
        data = self.run_salt("-C 'I%@companions%three%sarah*' test.ping")
        assert minion_in_returns("minion", data) is True
        assert minion_in_returns("sub_minion", data) is True

    @skipIf(True, "This test is unreliable. Need to investigate why more deeply.")
    @flaky
    def test_compound_pillar_pcre(self):
        data = self.run_salt("-C 'J%@knights%^(Lancelot|Galahad)$' test.ping")
        self.assertTrue(minion_in_returns("minion", data))
        self.assertTrue(minion_in_returns("sub_minion", data))

    def test_compound_nodegroup(self):
        data = self.run_salt('-C "N@multiline_nodegroup" test.ping')
        self.assertTrue(minion_in_returns("minion", data))
        self.assertTrue(minion_in_returns("sub_minion", data))
        data = self.run_salt('-C "N@multiline_nodegroup not sub_minion" test.ping')
        self.assertTrue(minion_in_returns("minion", data))
        self.assertFalse(minion_in_returns("sub_minion", data))
        data = self.run_salt(
            '-C "N@multiline_nodegroup not @fakenodegroup not sub_minion" test.ping'
        )
        self.assertTrue(minion_in_returns("minion", data))
        self.assertFalse(minion_in_returns("sub_minion", data))

    @slowTest
    def test_nodegroup(self):
        """
        test salt nodegroup matcher
        """
        data = self.run_salt("-N min test.ping")
        self.assertTrue(minion_in_returns("minion", data))
        self.assertFalse(minion_in_returns("sub_minion", data))
        time.sleep(2)
        data = self.run_salt("-N sub_min test.ping")
        self.assertFalse(minion_in_returns("minion", data))
        self.assertTrue(minion_in_returns("sub_minion", data))
        time.sleep(2)
        data = self.run_salt("-N mins test.ping")
        self.assertTrue(minion_in_returns("minion", data))
        self.assertTrue(minion_in_returns("sub_minion", data))
        time.sleep(2)
        data = self.run_salt("-N unknown_nodegroup test.ping")
        self.assertFalse(minion_in_returns("minion", data))
        self.assertFalse(minion_in_returns("sub_minion", data))
        time.sleep(2)
        data = self.run_salt("-N redundant_minions test.ping")
        self.assertTrue(minion_in_returns("minion", data))
        self.assertTrue(minion_in_returns("sub_minion", data))
        time.sleep(2)
        data = "\n".join(self.run_salt("-N nodegroup_loop_a test.ping"))
        self.assertIn("No minions matched", data)
        time.sleep(2)
        data = self.run_salt("-N multiline_nodegroup test.ping")
        self.assertTrue(minion_in_returns("minion", data))
        self.assertTrue(minion_in_returns("sub_minion", data))

    @slowTest
    def test_nodegroup_list(self):
        data = self.run_salt("-N list_group test.ping")
        self.assertTrue(minion_in_returns("minion", data))
        self.assertTrue(minion_in_returns("sub_minion", data))

        data = self.run_salt("-N list_group2 test.ping")
        self.assertTrue(minion_in_returns("minion", data))
        self.assertTrue(minion_in_returns("sub_minion", data))

        data = self.run_salt("-N one_list_group test.ping")
        self.assertTrue(minion_in_returns("minion", data))
        self.assertFalse(minion_in_returns("sub_minion", data))

        data = self.run_salt("-N one_minion_list test.ping")
        self.assertTrue(minion_in_returns("minion", data))
        self.assertFalse(minion_in_returns("sub_minion", data))

    @slowTest
    def test_glob(self):
        """
        test salt glob matcher
        """
        data = self.run_salt("minion test.ping")
        data = "\n".join(data)
        self.assertIn("minion", data)
        self.assertNotIn("sub_minion", data)
        data = self.run_salt('"*" test.ping')
        data = "\n".join(data)
        self.assertIn("minion", data)
        self.assertIn("sub_minion", data)

    @slowTest
    def test_regex(self):
        """
        test salt regex matcher
        """
        data = self.run_salt('-E "^minion$" test.ping')
        data = "\n".join(data)
        self.assertIn("minion", data)
        self.assertNotIn("sub_minion", data)
        data = self.run_salt('-E ".*" test.ping')
        data = "\n".join(data)
        self.assertIn("minion", data)
        self.assertIn("sub_minion", data)

    @slowTest
    def test_grain(self):
        """
        test salt grain matcher
        """
        # Sync grains
        self.run_salt('-t1 "*" saltutil.sync_grains')
        # First-level grain (string value)
        data = self.run_salt('-t 1 -G "test_grain:cheese" test.ping')
        data = "\n".join(data)
        self.assertIn("minion", data)
        self.assertNotIn("sub_minion", data)
        data = self.run_salt('-G "test_grain:spam" test.ping')
        data = "\n".join(data)
        self.assertIn("sub_minion", data)
        self.assertNotIn("minion", data.replace("sub_minion", "stub"))
        # Custom grain
        data = self.run_salt('-t 1 -G "match:maker" test.ping')
        data = "\n".join(data)
        self.assertIn("minion", data)
        self.assertIn("sub_minion", data)
        # First-level grain (list member)
        data = self.run_salt('-t 1 -G "planets:earth" test.ping')
        data = "\n".join(data)
        self.assertIn("minion", data)
        self.assertNotIn("sub_minion", data)
        data = self.run_salt('-G "planets:saturn" test.ping')
        data = "\n".join(data)
        self.assertIn("sub_minion", data)
        self.assertNotIn("minion", data.replace("sub_minion", "stub"))
        data = self.run_salt('-G "planets:pluto" test.ping')
        expect = None
        if self.master_opts["transport"] in ("zeromq", "tcp"):
            expect = (
                "No minions matched the target. "
                "No command was sent, no jid was "
                "assigned."
            )
        self.assertEqual("".join(data), expect)
        # Nested grain (string value)
        data = self.run_salt('-t 1 -G "level1:level2:foo" test.ping')
        data = "\n".join(data)
        self.assertIn("minion", data)
        self.assertNotIn("sub_minion", data)
        data = self.run_salt('-G "level1:level2:bar" test.ping')
        data = "\n".join(data)
        self.assertIn("sub_minion", data)
        self.assertNotIn("minion", data.replace("sub_minion", "stub"))
        # Nested grain (list member)
        data = self.run_salt('-t 1 -G "companions:one:ian" test.ping')
        data = "\n".join(data)
        self.assertIn("minion", data)
        self.assertNotIn("sub_minion", data)
        data = self.run_salt('-G "companions:two:jamie" test.ping')
        data = "\n".join(data)
        self.assertIn("sub_minion", data)
        self.assertNotIn("minion", data.replace("sub_minion", "stub"))
        # Test for issue: https://github.com/saltstack/salt/issues/19651
        data = self.run_salt('-G "companions:*:susan" test.ping')
        data = "\n".join(data)
        self.assertIn("minion:", data)
        self.assertNotIn("sub_minion", data)
        # Test to ensure wildcard at end works correctly
        data = self.run_salt('-G "companions:one:*" test.ping')
        data = "\n".join(data)
        self.assertIn("minion:", data)
        self.assertNotIn("sub_minion", data)
        # Test to ensure multiple wildcards works correctly
        data = self.run_salt('-G "companions:*:*" test.ping')
        data = "\n".join(data)
        self.assertIn("minion:", data)
        self.assertIn("sub_minion", data)

    @slowTest
    def test_regrain(self):
        """
        test salt grain matcher
        """
        data = self.run_salt('-t 1 --grain-pcre "test_grain:^cheese$" test.ping')
        data = "\n".join(data)
        self.assertIn("minion", data)
        self.assertNotIn("sub_minion", data)
        data = self.run_salt('--grain-pcre "test_grain:.*am$" test.ping')
        data = "\n".join(data)
        self.assertIn("sub_minion", data)
        self.assertNotIn("minion", data.replace("sub_minion", "stub"))

    @slowTest
    def test_pillar(self):
        """
        test pillar matcher
        """
        # First-level pillar (string value)
        data = self.run_salt('-I "monty:python" test.ping')
        data = "\n".join(data)
        self.assertIn("minion", data)
        self.assertIn("sub_minion", data)
        # First-level pillar (string value, only in sub_minion)
        data = self.run_salt('-I "sub:sub_minion" test.ping')
        data = "\n".join(data)
        self.assertIn("sub_minion", data)
        self.assertNotIn("minion", data.replace("sub_minion", "stub"))
        # First-level pillar (list member)
        data = self.run_salt('-I "knights:Bedevere" test.ping')
        data = "\n".join(data)
        self.assertIn("minion", data)
        self.assertIn("sub_minion", data)
        # Nested pillar (string value)
        data = self.run_salt('-I "level1:level2:foo" test.ping')
        data = "\n".join(data)
        self.assertIn("minion", data)
        self.assertIn("sub_minion", data)
        # Nested pillar (list member)
        data = self.run_salt('-I "companions:three:sarah jane" test.ping')
        data = "\n".join(data)
        self.assertIn("minion", data)
        self.assertIn("sub_minion", data)

    @slowTest
    def test_repillar(self):
        """
        test salt pillar PCRE matcher
        """
        data = self.run_salt('-J "monty:^(python|hall)$" test.ping')
        data = "\n".join(data)
        self.assertIn("minion", data)
        self.assertIn("sub_minion", data)
        data = self.run_salt('--pillar-pcre "knights:^(Robin|Lancelot)$" test.ping')
        data = "\n".join(data)
        self.assertIn("sub_minion", data)
        self.assertIn("minion", data.replace("sub_minion", "stub"))

    @slowTest
    def test_ipcidr(self):
        subnets_data = self.run_salt('--out yaml "*" network.subnets')
        yaml_data = salt.utils.yaml.safe_load("\n".join(subnets_data))

        # We're just after the first defined subnet from 'minion'
        subnet = yaml_data["minion"][0]

        data = self.run_salt("-S {0} test.ping".format(subnet))
        data = "\n".join(data)
        self.assertIn("minion", data)
        self.assertIn("sub_minion", data)

    @slowTest
    def test_static(self):
        """
        test salt static call
        """
        data = self.run_salt("minion test.ping --static")
        data = "\n".join(data)
        self.assertIn("minion", data)

    @slowTest
    def test_salt_documentation(self):
        """
        Test to see if we're supporting --doc
        """
        expect_to_find = "test.ping:"
        stdout, stderr = self.run_salt('-d "*" test', catch_stderr=True)
        error_msg = dedent(
            """
        Failed to find \'{expected}\' in output

        {sep}
        --- STDOUT -----
        {stdout}
        {sep}
        --- STDERR -----
        {stderr}
        {sep}
        """.format(
                sep="-" * 80,
                expected=expect_to_find,
                stdout="\n".join(stdout).strip(),
                stderr="\n".join(stderr).strip(),
            )
        )
        self.assertIn(expect_to_find, stdout, msg=error_msg)

    @slowTest
    def test_salt_documentation_too_many_arguments(self):
        """
        Test to see if passing additional arguments shows an error
        """
        data = self.run_salt(
            '-d minion salt ldap.search "filter=ou=People"', catch_stderr=True
        )
        self.assertIn(
            "You can only get documentation for one method at one time",
            "\n".join(data[1]),
        )



# -*- coding: utf-8 -*-
"""
    :codeauthor: :email:`Shane Lee <slee@saltstack.com>`
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.grains.pending_reboot as pending_reboot
from tests.support.mock import patch

# Import Salt Testing Libs
from tests.support.unit import TestCase


class PendingRebootGrainTestCase(TestCase):
    """
    Test cases for pending_reboot grain
    """

    def test_pending_reboot_false(self):
        with patch("salt.utils.win_system.get_pending_reboot", return_value=False):
            result = pending_reboot.pending_reboot()
            self.assertFalse(result["pending_reboot"])

    def test_pending_reboot_true(self):
        with patch("salt.utils.win_system.get_pending_reboot", return_value=True):
            result = pending_reboot.pending_reboot()
            self.assertTrue(result["pending_reboot"])



# -*- coding: utf-8 -*-
"""
    tests.unit.utils.scheduler.base
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""

from __future__ import absolute_import

import copy
import logging
import os

import salt.utils.platform
import salt.utils.schedule
from salt.modules.test import ping
from salt.utils.process import SubprocessList
from saltfactories.utils.processes.helpers import terminate_process
from tests.support.mixins import SaltReturnAssertsMixin
from tests.support.mock import MagicMock, patch
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import TestCase

log = logging.getLogger(__name__)


class SchedulerTestsBase(TestCase, SaltReturnAssertsMixin):
    """
    Validate the pkg module
    """

    @classmethod
    def setUpClass(cls):
        root_dir = os.path.join(RUNTIME_VARS.TMP, "schedule-unit-tests")
        sock_dir = os.path.join(root_dir, "test-socks")

        default_config = salt.config.minion_config(None)
        default_config["conf_dir"] = root_dir
        default_config["root_dir"] = root_dir
        default_config["sock_dir"] = sock_dir
        default_config["pki_dir"] = os.path.join(root_dir, "pki")
        default_config["cachedir"] = os.path.join(root_dir, "cache")
        cls.default_config = default_config
        cls.subprocess_list = SubprocessList()

    @classmethod
    def tearDownClass(cls):
        del cls.default_config
        del cls.subprocess_list

    def setUp(self):
        with patch("salt.utils.schedule.clean_proc_dir", MagicMock(return_value=None)):
            functions = {"test.ping": ping}
            self.schedule = salt.utils.schedule.Schedule(
                copy.deepcopy(self.default_config),
                functions,
                returners={},
                new_instance=True,
            )
        self.schedule._subprocess_list = self.subprocess_list

    def tearDown(self):
        subprocess_list = self.subprocess_list
        processes = subprocess_list.processes
        self.schedule.reset()
        del self.schedule
        for proc in processes:
            if proc.is_alive():
                terminate_process(proc.pid, kill_children=True, slow_stop=True)
        subprocess_list.cleanup()
        processes = subprocess_list.processes
        if processes:
            for proc in processes:
                if proc.is_alive():
                    terminate_process(proc.pid, kill_children=True, slow_stop=False)
            subprocess_list.cleanup()
        processes = subprocess_list.processes
        if processes:
            log.warning("Processes left running: %s", processes)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.modules.mac_keychain as keychain

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class KeychainTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        return {keychain: {}}

    def test_install_cert(self):
        """
            Test installing a certificate into the macOS keychain
        """
        mock = MagicMock()
        with patch.dict(keychain.__salt__, {"cmd.run": mock}):
            keychain.install("/path/to/cert.p12", "passw0rd")
            mock.assert_called_once_with(
                "security import /path/to/cert.p12 -P passw0rd "
                "-k /Library/Keychains/System.keychain"
            )

    def test_install_cert_extras(self):
        """
            Test installing a certificate into the macOS keychain with extras
        """
        mock = MagicMock()
        with patch.dict(keychain.__salt__, {"cmd.run": mock}), patch(
            "salt.modules.mac_keychain.unlock_keychain"
        ) as unlock_mock:
            keychain.install(
                "/path/to/cert.p12",
                "passw0rd",
                "/path/to/chain",
                allow_any=True,
                keychain_password="passw0rd1",
            )
            unlock_mock.assert_called_once_with("/path/to/chain", "passw0rd1")
            mock.assert_called_once_with(
                "security import /path/to/cert.p12 -P passw0rd -k /path/to/chain -A"
            )

    def test_uninstall_cert(self):
        """
            Test uninstalling a certificate from the macOS keychain
        """
        mock = MagicMock()
        with patch.dict(keychain.__salt__, {"cmd.run": mock}):
            keychain.uninstall("/path/to/cert.p12", "passw0rd")
            mock.assert_called_once_with(
                'security delete-certificate -c "/path/to/cert.p12" passw0rd'
            )

    def test_list_certs(self):
        """
            Test listing available certificates in a keychain
        """
        expected = ["com.apple.systemdefault", "com.apple.kerberos.kdc"]
        mock = MagicMock(
            return_value='"com.apple.systemdefault"\n"com.apple.kerberos.kdc"'
        )
        with patch.dict(keychain.__salt__, {"cmd.run": mock}):
            out = keychain.list_certs("/path/to/cert.p12")
            mock.assert_called_once_with(
                "security find-certificate -a /path/to/cert.p12 | "
                'grep -o "alis".*\\" | grep -o \'\\"[-A-Za-z0-9.:() ]*\\"\'',
                python_shell=True,
            )

            self.assertEqual(out, expected)

    def test_get_friendly_name(self):
        """
            Test getting the friendly name of a certificate
        """
        expected = "ID Installer Salt"
        mock = MagicMock(return_value="friendlyName: ID Installer Salt")
        with patch.dict(keychain.__salt__, {"cmd.run": mock}):
            out = keychain.get_friendly_name("/path/to/cert.p12", "passw0rd")
            mock.assert_called_once_with(
                "openssl pkcs12 -in /path/to/cert.p12 -passin pass:passw0rd -info "
                "-nodes -nokeys 2> /dev/null | grep friendlyName:",
                python_shell=True,
            )

            self.assertEqual(out, expected)

    def test_get_default_keychain(self):
        """
            Test getting the default keychain
        """
        mock = MagicMock()
        with patch.dict(keychain.__salt__, {"cmd.run": mock}):
            keychain.get_default_keychain("frank", "system")
            mock.assert_called_once_with(
                "security default-keychain -d system", runas="frank"
            )

    def test_set_default_keychain(self):
        """
            Test setting the default keychain
        """
        mock = MagicMock()
        with patch.dict(keychain.__salt__, {"cmd.run": mock}):
            keychain.set_default_keychain("/path/to/chain.keychain", "system", "frank")
            mock.assert_called_once_with(
                "security default-keychain -d system -s /path/to/chain.keychain",
                runas="frank",
            )

    def test_unlock_keychain(self):
        """
            Test unlocking the keychain
        """
        mock = MagicMock()
        with patch.dict(keychain.__salt__, {"cmd.run": mock}):
            keychain.unlock_keychain("/path/to/chain.keychain", "passw0rd")
            mock.assert_called_once_with(
                "security unlock-keychain -p passw0rd /path/to/chain.keychain"
            )



# -*- coding: utf-8 -*-
"""
Test AnsibleGate State Module
"""
from __future__ import absolute_import, print_function, unicode_literals

# Import python libraries
import os
import shutil
import tempfile

# Import salt libraries
import salt.utils.files
import salt.utils.path
import yaml

# Import testing libraries
from tests.support.case import ModuleCase
from tests.support.helpers import (
    destructiveTest,
    flaky,
    requires_sshd_server,
    requires_system_grains,
)
from tests.support.mixins import SaltReturnAssertsMixin
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import SkipTest, skipIf


@destructiveTest
@requires_sshd_server
@skipIf(
    not salt.utils.path.which("ansible-playbook"), "ansible-playbook is not installed"
)
class AnsiblePlaybooksTestCase(ModuleCase, SaltReturnAssertsMixin):
    """
    Test ansible.playbooks states
    """

    @classmethod
    @requires_system_grains
    def setUpClass(cls, grains=None):  # pylint: disable=arguments-differ
        if grains.get("os_family") == "RedHat" and grains.get("osmajorrelease") == 6:
            raise SkipTest(
                "This test hangs the test suite on RedHat 6. Skipping for now."
            )

    def setUp(self):
        priv_file = os.path.join(RUNTIME_VARS.TMP_CONF_DIR, "key_test")
        data = {
            "all": {
                "hosts": {
                    "localhost": {
                        "ansible_host": "127.0.0.1",
                        "ansible_port": 2827,
                        "ansible_user": RUNTIME_VARS.RUNNING_TESTS_USER,
                        "ansible_ssh_private_key_file": priv_file,
                        "ansible_ssh_extra_args": (
                            "-o StrictHostKeyChecking=false "
                            "-o UserKnownHostsFile=/dev/null "
                        ),
                    },
                },
            },
        }
        self.tempdir = tempfile.mkdtemp()
        self.inventory = self.tempdir + "inventory"
        with salt.utils.files.fopen(self.inventory, "w") as yaml_file:
            yaml.dump(data, yaml_file, default_flow_style=False)

    def tearDown(self):
        shutil.rmtree(self.tempdir)
        delattr(self, "tempdir")
        delattr(self, "inventory")

    @flaky
    def test_ansible_playbook(self):
        ret = self.run_state(
            "ansible.playbooks",
            name="remove.yml",
            git_repo="git://github.com/gtmanfred/playbooks.git",
            ansible_kwargs={"inventory": self.inventory},
        )
        self.assertSaltTrueReturn(ret)
        ret = self.run_state(
            "ansible.playbooks",
            name="install.yml",
            git_repo="git://github.com/gtmanfred/playbooks.git",
            ansible_kwargs={"inventory": self.inventory},
        )
        self.assertSaltTrueReturn(ret)
